package net.geant.wifimon.agent.service;

import net.geant.wifimon.agent.model.UserChangePasswordFormModel;
import net.geant.wifimon.agent.model.UserCreateFormModel;
import net.geant.wifimon.agent.repository.UserRepository;
import net.geant.wifimon.model.entity.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.Optional;

/**
 * Created by kanakisn on 4/24/16.
 */
@Service
public class UserServiceImpl implements UserService {

    private final UserRepository userRepository;

    @Autowired
    public UserServiceImpl(final UserRepository userRepository) {
        this.userRepository = userRepository;
    }

    @Override
    public Optional<User> getUserById(long id) {
        return Optional.ofNullable(userRepository.findById(id).orElse(null));
    }

    @Override
    public Optional<User> getUserByEmail(String email) {
        return userRepository.findOneByEmail(email);
    }

    @Override
    public Collection<User> getAllUsers() {
        return userRepository.findAll(Sort.by("email"));
    }

    @Override
    public User create(UserCreateFormModel form) {
        User user = new User();
        user.setEmail(form.getEmail());
        user.setPasswordHash(new BCryptPasswordEncoder().encode(form.getPassword()));
        user.setRole(form.getRole());
        return userRepository.save(user);
    }

    @Override
    public User changePassword(UserChangePasswordFormModel form) {
                String userEmail = SecurityContextHolder.getContext().getAuthentication().getName();
                User user = this.getUserByEmail(userEmail).orElse(null);
                user.setPasswordHash(new BCryptPasswordEncoder().encode(form.getPassword()));
                return userRepository.save(user);
    }

    @Override
    public void delete(Long id) {
        userRepository.deleteById(id);
    }
}
