package net.geant.wifimon.agent.controller;

import net.geant.wifimon.agent.repository.AccesspointRepository;
import net.geant.wifimon.agent.repository.UserRepository;
import net.geant.wifimon.agent.repository.VisualOptionsRepository;
import net.geant.wifimon.model.entity.UrlParameters;
import net.geant.wifimon.model.entity.UserData;
import net.geant.wifimon.model.entity.UserVisualOption;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpSession;

/**
 * Created by kanakisn on 8/5/15.
 */

@Controller
public class MeasurementsController {

    private static final String SERVER_HOST_NAME = "server.host.name";
    private static final String KIBANA_PROTOCOL = "kibana.protocol";
    private static final String KIBANA_PORT = "kibana.port";
    private static final String CLASS_SETTINGS_STATE = "active";
    private static final String ELASTICSEARCH_URL = "elasticsearchURL";

    @Autowired
    AccesspointRepository accesspointRepository;

    @Autowired
    VisualOptionsRepository visualOptionsRepository;

    @Autowired
    UserRepository userRepository;

    @Autowired
    Environment environment;

    @GetMapping(value = "/username")
    @ResponseBody
    public String currentUserName(Authentication authentication) {
        return authentication.getName();
    }

    @GetMapping(value = "/secure/overview")
    public String elasticsearchOverview(Model model, HttpSession session, HttpServletRequest request) {

        UrlParameters urlParameters = createUrlParameters(request);

        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
                "/app/kibana#/dashboard/wifimon_dash_01_v0.1?embed=true&_g=(refreshInterval:(display:'30+seconds',pause:!f,section:1,value:30000),time:(from:now%2Fd,mode:quick,to:now%2Fd))&_a=(description:'',filters:!(),options:(darkTheme:!f),panels:!((col:1,id:wifimon_vis_44_v0.1,panelIndex:1,row:1,size_x:12,size_y:6,type:visualization)),query:(" + urlParameters.getQueryFilter() + "),timeRestore:!t,title:'WiFiMon:+Overview+DashBoard+%5Bv0.1%5D',uiState:(P-1:(vis:(defaultColors:('0+-+100':'rgb(0,104,55)')))),viewMode:view)";

        model.addAttribute("classActiveSettingsElasticOverview", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchOverviewDashboard";
    }

    @GetMapping(value = "/secure/measurements")
    public String genericMeasurements(Model model, HttpSession session, HttpServletRequest request) {

        UrlParameters urlParameters = createUrlParameters(request);
        String elasticsearchURL;

        if (visualOptionsRepository.getLastEntry() != null) {
            if (visualOptionsRepository.getLastEntry().getUserdata().equals(UserData.HIDE)) {
                elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
                        "/app/kibana#/dashboard/wifimon_privacy_dash_02_v0.1?embed=true&_g=(refreshInterval:(display:'30+seconds',pause:!f,section:1,value:30000),time:(from:now%2Fd,mode:quick,to:now%2Fd))&_a=(description:'',filters:!(),options:(darkTheme:!f),panels:!((col:1,id:wifimon_privacy_vis_43_v0.1,panelIndex:1,row:1,size_x:12,size_y:7,type:visualization)),query:(" + urlParameters.getQueryFilter() + "),timeRestore:!t,title:'WiFiMon:+Recent+measurements+%5Bv0.1-privacy%5D',uiState:(P-1:(vis:(params:(sort:(columnIndex:!n,direction:!n))))),viewMode:view)";
            } else {
                elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
                        "/app/kibana#/dashboard/wifimon_dash_02_v0.1?embed=true&_g=(refreshInterval:(display:'30+seconds',pause:!f,section:1,value:30000),time:(from:now%2Fd,mode:quick,to:now%2Fd))&_a=(description:'',filters:!(),options:(darkTheme:!f),panels:!((col:1,id:wifimon_vis_43_v0.1,panelIndex:1,row:1,size_x:12,size_y:8,type:visualization)),query:(" + urlParameters.getQueryFilter() + "),timeRestore:!t,title:'WiFiMon:+Recent+measurements+%5Bv0.1%5D',uiState:(P-1:(vis:(params:(sort:(columnIndex:!n,direction:!n))))),viewMode:view)";
            }
        } else {
            elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
                    "/app/kibana#/dashboard/wifimon_dash_02_v0.1?embed=true&_g=(refreshInterval:(display:'30+seconds',pause:!f,section:1,value:30000),time:(from:now%2Fd,mode:quick,to:now%2Fd))&_a=(description:'',filters:!(),options:(darkTheme:!f),panels:!((col:1,id:wifimon_vis_43_v0.1,panelIndex:1,row:1,size_x:12,size_y:8,type:visualization)),query:(" + urlParameters.getQueryFilter() + "),timeRestore:!t,title:'WiFiMon:+Recent+measurements+%5Bv0.1%5D',uiState:(P-1:(vis:(params:(sort:(columnIndex:!n,direction:!n))))),viewMode:view)";
        }

        model.addAttribute("classActiveSettingsElasticMeasurements", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/genericMeasurements";
    }

       @GetMapping(value = "/secure/crowdsourced/averageValues")
    public String elasticsearchCrowdsourcedAverage(Model model, HttpSession session, HttpServletRequest request)
    {
            String elasticsearchURL;
            elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		    "/app/dashboards#/view/eda78290-20e1-11ea-9567-dfd517a2bd97?embed=true&_g=(filters:!())&_a=(description:'A%20dashboard%20related%20to%20average%20values%20of%20Crowdsourced%20Measurements',filters:!(('$state':(store:appState),meta:(alias:!n,disabled:!f,index:wifimon_v0.1,key:Origin,negate:!f,params:(query:User),type:phrase),query:(match_phrase:(Origin:User)))),fullScreenMode:!f,options:(darkTheme:!f,hidePanelTitles:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:(),hidePanelTitles:!f,vis:(legendOpen:!f)),gridData:(h:15,i:'1',w:24,x:0,y:0),id:wifimon_vis_13_v0.1,panelIndex:'1',title:'Average%20Download%20Throughput%20per%20Client%20IP%20(Bottom%2010%20Clients)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,vis:(legendOpen:!f)),gridData:(h:15,i:'2',w:24,x:24,y:0),id:wifimon_vis_38_v0.1,panelIndex:'2',title:'Average%20Download%20Throughput%20per%20Client%20IP%20(Top%2010%20Clients)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,vis:(legendOpen:!f)),gridData:(h:15,i:'3',w:24,x:0,y:15),id:wifimon_vis_33_v0.1,panelIndex:'3',title:'Average%20Upload%20Throughput%20per%20Client%20IP%20(Bottom%2010%20Clients)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,vis:(legendOpen:!f)),gridData:(h:15,i:'4',w:24,x:24,y:15),id:wifimon_vis_32_v0.1,panelIndex:'4',title:'Average%20Upload%20Throughput%20per%20Client%20IP%20(Top%2010%20Clients)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,vis:(legendOpen:!f)),gridData:(h:15,i:'5',w:24,x:0,y:30),id:wifimon_vis_18_v0.1,panelIndex:'5',title:'Average%20HTTP%20Ping%20Round%20Trip%20Time%20per%20Client%20IP%20(Bottom%2010%20Clients)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,vis:(legendOpen:!f)),gridData:(h:15,i:'6',w:24,x:24,y:30),id:wifimon_vis_17_v0.1,panelIndex:'6',title:'Average%20HTTP%20Ping%20Round%20Trip%20Time%20per%20Client%20IP%20(Top%2010%20Clients)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'7',w:24,x:0,y:45),id:wifimon_vis_05_v0.1,panelIndex:'7',title:'Average%20Download%20Throughput%20per%20Access%20Point%20(AP)%20MAC%20(Bottom%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'8',w:24,x:24,y:45),id:wifimon_vis_40_v0.1,panelIndex:'8',title:'Average%20Download%20Throughput%20per%20Access%20Point%20(AP)%20MAC%20(Top%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'10',w:24,x:24,y:60),id:wifimon_vis_25_v0.1,panelIndex:'10',title:'Average%20Upload%20Throughput%20per%20Access%20Point%20(AP)%20MAC%20(Top%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,spy:(mode:(fill:!f,name:table)),vis:(legendOpen:!f)),gridData:(h:15,i:'11',w:24,x:0,y:120),id:wifimon_vis_28_v0.1,panelIndex:'11',title:'Average%20HTTP%20Ping%20Round%20Trip%20Time%20per%20Client%20MAC%20(Bottom%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'13',w:24,x:0,y:135),id:wifimon_vis_15_v0.1,panelIndex:'13',title:'Average%20Download%20Throughput%20per%20Operating%20System%20(OS)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'14',w:24,x:24,y:135),id:wifimon_vis_39_v0.1,panelIndex:'14',title:'Average%20Download%20Throughput%20per%20Client%20Browser',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'15',w:24,x:0,y:150),id:wifimon_vis_24_v0.1,panelIndex:'15',title:'Average%20Upload%20Throughput%20per%20Operating%20System%20(OS)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'16',w:24,x:24,y:165),id:wifimon_vis_20_v0.1,panelIndex:'16',title:'Average%20HTTP%20Ping%20Round%20Trip%20Time%20per%20Client%20Browser',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'17',w:24,x:0,y:165),id:wifimon_vis_21_v0.1,panelIndex:'17',title:'Average%20HTTP%20Ping%20Round%20Trip%20Time%20per%20Operating%20System%20(OS)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'19',w:24,x:0,y:60),id:wifimon_vis_22_v0.1,panelIndex:'19',title:'Average%20Upload%20Throughput%20per%20Access%20Point%20(AP)%20MAC%20(Bottom%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'24',w:24,x:24,y:150),id:wifimon_vis_23_v0.1,panelIndex:'24',title:'Average%20Upload%20Throughput%20per%20Client%20Browser',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'26',w:24,x:0,y:75),id:wifimon_vis_41_v0.1,panelIndex:'26',title:'Average%20HTTP%20Ping%20Round%20Trip%20Time%20per%20Access%20Point%20(AP)%20MAC%20(Bottom%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'27',w:24,x:24,y:75),id:wifimon_vis_34_v0.1,panelIndex:'27',title:'Average%20HTTP%20Ping%20Round%20Trip%20Time%20per%20Access%20Point%20(AP)%20MAC%20(Top%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,spy:(mode:(fill:!f,name:table)),table:!n,vis:(legendOpen:!f)),gridData:(h:15,i:'28',w:24,x:0,y:90),id:wifimon_vis_12_v0.1,panelIndex:'28',title:'Average%20Download%20Throughput%20per%20Client%20MAC%20(Bottom%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,spy:(mode:(fill:!f,name:table)),vis:(legendOpen:!f)),gridData:(h:15,i:'29',w:24,x:24,y:90),id:wifimon_vis_11_v0.1,panelIndex:'29',title:'Average%20Download%20Throughput%20per%20Client%20MAC%20(Top%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,spy:(mode:(fill:!f,name:table)),table:!n,vis:(legendOpen:!f)),gridData:(h:15,i:'30',w:24,x:0,y:105),id:wifimon_vis_31_v0.1,panelIndex:'30',title:'Average%20Upload%20Throughput%20per%20Client%20MAC%20(Bottom%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,spy:(mode:(fill:!f,name:table)),vis:(legendOpen:!f)),gridData:(h:15,i:'31',w:24,x:24,y:105),id:wifimon_vis_30_v0.1,panelIndex:'31',title:'Average%20Upload%20Throughput%20per%20Client%20MAC%20(Top%2010)',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f,spy:(mode:(fill:!f,name:table)),vis:(legendOpen:!f)),gridData:(h:15,i:'32',w:24,x:24,y:120),id:wifimon_vis_27_v0.1,panelIndex:'32',title:'Average%20HTTP%20Ping%20Round%20Trip%20Time%20per%20Client%20MAC%20(Top%2010)',type:visualization,version:'7.16.2')),query:(language:lucene,query:''),tags:!(),timeRestore:!t,title:'Crowdsourced%20Measurements%20Dashboard%20(Average)',viewMode:view)";
             model.addAttribute("classActiveSettingsCrowdsourced", CLASS_SETTINGS_STATE);
             model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
             return "secure/elasticsearchSubnetsAverage";
    }

    @GetMapping(value = "/secure/crowdsourced/medianValues")
    public String elasticsearchCrowdsourcedMedian(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/4978de50-3dac-11ec-8eec-09df95559c81?embed=true&_g=(filters:!())&_a=(description:'A%20dashboard%20related%20to%20median%20values%20of%20Crowdsourced%20Measurements',filters:!(('$state':(store:appState),meta:(alias:!n,disabled:!f,index:wifimon_v0.1,key:Origin,negate:!f,params:(query:User),type:phrase),query:(match_phrase:(Origin:User)))),fullScreenMode:!f,options:(hidePanelTitles:!f,syncColors:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:f7122f1c-d0b7-44a8-967b-f129ce79dd7b,w:24,x:0,y:0),id:'05a40d80-3d9d-11ec-8eec-09df95559c81',panelIndex:f7122f1c-d0b7-44a8-967b-f129ce79dd7b,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:'9c290e97-0b62-4e4b-bf0f-b4193a3df3be',w:24,x:24,y:0),id:'33d241e0-3d9d-11ec-8eec-09df95559c81',panelIndex:'9c290e97-0b62-4e4b-bf0f-b4193a3df3be',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:'66fd46c6-2dab-4681-ad29-c9ecdc38d066',w:24,x:0,y:15),id:wifimon_vis_33_v0.1,panelIndex:'66fd46c6-2dab-4681-ad29-c9ecdc38d066',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:b71c535a-c284-459e-afd2-59d7fe5bd6aa,w:24,x:24,y:15),id:'1e010340-3d9f-11ec-8eec-09df95559c81',panelIndex:b71c535a-c284-459e-afd2-59d7fe5bd6aa,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:'3aa583cc-541d-4eb6-ad5f-23e617aef770',w:24,x:24,y:30),id:f91908d0-3d9d-11ec-8eec-09df95559c81,panelIndex:'3aa583cc-541d-4eb6-ad5f-23e617aef770',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'6ef837ce-a86a-460c-b755-138b6f4c20fe',w:24,x:0,y:45),id:e9a63b90-3da0-11ec-8eec-09df95559c81,panelIndex:'6ef837ce-a86a-460c-b755-138b6f4c20fe',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:d021d1c0-d64c-403d-b8d2-8b7a10e4daaa,w:24,x:24,y:45),id:'28be6c80-3da1-11ec-8eec-09df95559c81',panelIndex:d021d1c0-d64c-403d-b8d2-8b7a10e4daaa,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:cab16151-b407-4559-857c-926ba4b9b7e2,w:24,x:0,y:60),id:'9dbb0070-3da1-11ec-8eec-09df95559c81',panelIndex:cab16151-b407-4559-857c-926ba4b9b7e2,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'9ae1f72d-2988-4e8a-b447-2be3b5121510',w:24,x:24,y:60),id:c619c790-3da1-11ec-8eec-09df95559c81,panelIndex:'9ae1f72d-2988-4e8a-b447-2be3b5121510',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'3635f384-5f40-436f-aaa2-8d46284db23b',w:24,x:0,y:75),id:b7be48a0-3da2-11ec-8eec-09df95559c81,panelIndex:'3635f384-5f40-436f-aaa2-8d46284db23b',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:be0f7c9a-c86b-4a7f-af9c-ee977c5004ac,w:24,x:24,y:75),id:fe08f940-3da2-11ec-8eec-09df95559c81,panelIndex:be0f7c9a-c86b-4a7f-af9c-ee977c5004ac,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:fb513777-3403-46bf-af18-137fb1b104a3,w:24,x:0,y:90),id:'3b7336a0-3da4-11ec-8eec-09df95559c81',panelIndex:fb513777-3403-46bf-af18-137fb1b104a3,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:'0692ac2d-5a3f-404b-9267-c814e7f87855',w:24,x:24,y:90),id:'5cae2410-3da4-11ec-8eec-09df95559c81',panelIndex:'0692ac2d-5a3f-404b-9267-c814e7f87855',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:f807fdeb-57e7-4d17-806e-171b9047cd8c,w:24,x:0,y:105),id:'03442ae0-3da5-11ec-8eec-09df95559c81',panelIndex:f807fdeb-57e7-4d17-806e-171b9047cd8c,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:a1672b8b-5a34-4207-9c29-88c11818df8b,w:24,x:24,y:105),id:e5538670-3da4-11ec-8eec-09df95559c81,panelIndex:a1672b8b-5a34-4207-9c29-88c11818df8b,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:'4b711746-671f-4e1a-8da9-29575158bd4a',w:24,x:0,y:120),id:'9c5224d0-3da5-11ec-8eec-09df95559c81',panelIndex:'4b711746-671f-4e1a-8da9-29575158bd4a',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:(),vis:(legendOpen:!f)),gridData:(h:15,i:'65b5a9eb-5db7-40c9-8bd7-d88a7c41586c',w:24,x:24,y:120),id:c61a18e0-3da5-11ec-8eec-09df95559c81,panelIndex:'65b5a9eb-5db7-40c9-8bd7-d88a7c41586c',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:e5c40148-893d-4ed9-b692-4df5043e257b,w:24,x:0,y:135),id:'0e684530-3da7-11ec-8eec-09df95559c81',panelIndex:e5c40148-893d-4ed9-b692-4df5043e257b,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'6ef4b89a-c86b-4a1b-bcf5-508ff9083061',w:24,x:24,y:135),id:'08459260-3da8-11ec-8eec-09df95559c81',panelIndex:'6ef4b89a-c86b-4a1b-bcf5-508ff9083061',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'584bee6e-4098-481d-afc3-30775fcb7b96',w:24,x:0,y:150),id:'7ecfc0a0-3da7-11ec-8eec-09df95559c81',panelIndex:'584bee6e-4098-481d-afc3-30775fcb7b96',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:c61cdafd-c0f8-4545-8959-1bf634305a36,w:24,x:24,y:150),id:'3037dc10-3da8-11ec-8eec-09df95559c81',panelIndex:c61cdafd-c0f8-4545-8959-1bf634305a36,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'7edc33c6-bd89-4c5c-ab33-06e0f9dc90a7',w:24,x:0,y:165),id:'4c73e820-3da7-11ec-8eec-09df95559c81',panelIndex:'7edc33c6-bd89-4c5c-ab33-06e0f9dc90a7',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'440d9122-68ef-4b5c-8410-46140bfe9a59',w:24,x:24,y:165),id:'5a0c7a50-3da8-11ec-8eec-09df95559c81',panelIndex:'440d9122-68ef-4b5c-8410-46140bfe9a59',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'5ad10681-3bdf-4435-81d4-bad938ea4df3',w:24,x:0,y:30),id:'9a4cbe40-3dad-11ec-8eec-09df95559c81',panelIndex:'5ad10681-3bdf-4435-81d4-bad938ea4df3',type:visualization,version:'7.16.2')),query:(language:kuery,query:''),tags:!(),timeRestore:!t,title:'Crowdsourced%20Measurements%20Dashboard%20(Median)',viewMode:view)";
        model.addAttribute("classActiveSettingsCrowdsourced", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchSubnetsMedian";
    }
    
    @GetMapping(value = "/secure/crowdsourced/maxValues")
    public String elasticsearchCrowdsourcedMax(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/8ee60670-6d9f-11ec-bbd5-2b486ded5a8c?embed=true&_g=(filters:!())&_a=(description:'A%20dashboard%20related%20to%20max%20values%20of%20Crowdsourced%20Measurements',filters:!(('$state':(store:appState),meta:(alias:!n,disabled:!f,index:wifimon_v0.1,key:Origin,negate:!f,params:(query:User),type:phrase),query:(match_phrase:(Origin:User)))),fullScreenMode:!f,options:(hidePanelTitles:!f,syncColors:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:()),gridData:(h:15,i:'7930c26e-572a-49fd-8c9f-7d3bcdd0a7de',w:24,x:0,y:0),id:df498b70-6d6b-11ec-bbd5-2b486ded5a8c,panelIndex:'7930c26e-572a-49fd-8c9f-7d3bcdd0a7de',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:c744cd69-8fdb-4e4d-b719-727d05e95e90,w:24,x:24,y:0),id:'2ede9f40-6d6c-11ec-bbd5-2b486ded5a8c',panelIndex:c744cd69-8fdb-4e4d-b719-727d05e95e90,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'38aa7d68-6441-481a-80da-2b631f59494a',w:24,x:0,y:15),id:'7e24ee70-6d7a-11ec-bbd5-2b486ded5a8c',panelIndex:'38aa7d68-6441-481a-80da-2b631f59494a',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:b135bf7f-cd26-4156-8e1c-adf9a8735bad,w:24,x:24,y:15),id:fc2c2b30-6d7f-11ec-bbd5-2b486ded5a8c,panelIndex:b135bf7f-cd26-4156-8e1c-adf9a8735bad,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:c3322328-51fd-4d20-b635-85482a50ce0a,w:24,x:0,y:30),id:'1ddd4d50-6d70-11ec-bbd5-2b486ded5a8c',panelIndex:c3322328-51fd-4d20-b635-85482a50ce0a,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'301f6a3b-871a-48de-8bc0-8d119f7f422a',w:24,x:24,y:30),id:'65fa25e0-6d70-11ec-bbd5-2b486ded5a8c',panelIndex:'301f6a3b-871a-48de-8bc0-8d119f7f422a',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'29ea35bd-61f4-4375-9833-793ee6f55198',w:24,x:0,y:45),id:fea63870-6d6a-11ec-bbd5-2b486ded5a8c,panelIndex:'29ea35bd-61f4-4375-9833-793ee6f55198',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'4389c06a-8d27-49b1-bc29-d1040eb703ce',w:24,x:24,y:45),id:'71f3bcd0-6d6b-11ec-bbd5-2b486ded5a8c',panelIndex:'4389c06a-8d27-49b1-bc29-d1040eb703ce',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'7cc387ea-8ff6-4636-81a9-82d5032e534f',w:24,x:0,y:60),id:'65c93bc0-6d79-11ec-bbd5-2b486ded5a8c',panelIndex:'7cc387ea-8ff6-4636-81a9-82d5032e534f',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'61bc7638-1eef-4b86-9f54-0c863609f923',w:24,x:24,y:60),id:c6fe7cc0-6d79-11ec-bbd5-2b486ded5a8c,panelIndex:'61bc7638-1eef-4b86-9f54-0c863609f923',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'3f06ae84-fda3-40e8-8439-e8dc90df95b5',w:24,x:0,y:75),id:c834b3d0-6d6e-11ec-bbd5-2b486ded5a8c,panelIndex:'3f06ae84-fda3-40e8-8439-e8dc90df95b5',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'0bf00112-87f9-4257-bb29-aa71e665c06e',w:24,x:24,y:75),id:'16f47640-6d6f-11ec-bbd5-2b486ded5a8c',panelIndex:'0bf00112-87f9-4257-bb29-aa71e665c06e',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'060c7a6c-5fc2-446d-b4a8-128913156728',w:24,x:0,y:90),id:'41d98500-6d6d-11ec-bbd5-2b486ded5a8c',panelIndex:'060c7a6c-5fc2-446d-b4a8-128913156728',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:d6c68b25-c688-4d48-8e4c-3f7d77cf3b3c,w:24,x:24,y:90),id:'98181260-6d6d-11ec-bbd5-2b486ded5a8c',panelIndex:d6c68b25-c688-4d48-8e4c-3f7d77cf3b3c,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:a8797b8f-3544-4a92-9d6c-821089686e4e,w:24,x:0,y:105),id:'3655bc80-6d81-11ec-bbd5-2b486ded5a8c',panelIndex:a8797b8f-3544-4a92-9d6c-821089686e4e,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'010de062-0640-4168-9c3f-1fff4e895bb1',w:24,x:24,y:105),id:'1d62e800-6d82-11ec-bbd5-2b486ded5a8c',panelIndex:'010de062-0640-4168-9c3f-1fff4e895bb1',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'0a3f708c-a75e-4781-8d9e-7493897d3255',w:24,x:0,y:120),id:'25d928c0-6d76-11ec-bbd5-2b486ded5a8c',panelIndex:'0a3f708c-a75e-4781-8d9e-7493897d3255',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'68d3f74d-b291-4c26-a9a9-1465f863c82b',w:24,x:24,y:120),id:'7f6e5b30-6d76-11ec-bbd5-2b486ded5a8c',panelIndex:'68d3f74d-b291-4c26-a9a9-1465f863c82b',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:e0be29e0-32f0-43b4-ac09-edb0a08982b3,w:24,x:0,y:135),id:'52c01aa0-6d77-11ec-bbd5-2b486ded5a8c',panelIndex:e0be29e0-32f0-43b4-ac09-edb0a08982b3,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'9809f9cc-8ffe-4a10-aa1e-3c877c84522b',w:24,x:24,y:135),id:'8e9a3560-6d68-11ec-bbd5-2b486ded5a8c',panelIndex:'9809f9cc-8ffe-4a10-aa1e-3c877c84522b',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'9e7b1f77-10e8-4986-9e92-14ac3880436e',w:24,x:0,y:150),id:d88b8200-6d77-11ec-bbd5-2b486ded5a8c,panelIndex:'9e7b1f77-10e8-4986-9e92-14ac3880436e',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'52a2b7a2-02bd-43fc-9d3f-dd0ab0fca41c',w:24,x:24,y:150),id:'2dff74c0-6d6a-11ec-bbd5-2b486ded5a8c',panelIndex:'52a2b7a2-02bd-43fc-9d3f-dd0ab0fca41c',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:e9253005-83b5-479d-a080-b71cc413947b,w:24,x:0,y:165),id:'36c9f130-6d78-11ec-bbd5-2b486ded5a8c',panelIndex:e9253005-83b5-479d-a080-b71cc413947b,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:e23dffac-31ea-4eba-993d-0236a08e370b,w:24,x:24,y:165),id:'81985d90-6d6a-11ec-bbd5-2b486ded5a8c',panelIndex:e23dffac-31ea-4eba-993d-0236a08e370b,type:visualization,version:'7.16.2')),query:(language:kuery,query:''),tags:!(),timeRestore:!t,title:'Crowdsourced%20Measurements%20Dashboard%20(Max)',viewMode:view)";
        model.addAttribute("classActiveSettingsCrowdsourced", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchSubnetsMax";
    }
    
    @GetMapping(value = "/secure/crowdsourced/minValues")
    public String elasticsearchCrowdsourcedMin(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/a3a311c0-6da4-11ec-bbd5-2b486ded5a8c?embed=true&_g=(filters:!())&_a=(description:'A%20dashboard%20related%20to%20min%20values%20of%20Crowdsourced%20Measurements',filters:!(('$state':(store:appState),meta:(alias:!n,disabled:!f,index:wifimon_v0.1,key:Origin,negate:!f,params:(query:User),type:phrase),query:(match_phrase:(Origin:User)))),fullScreenMode:!f,options:(hidePanelTitles:!f,syncColors:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:()),gridData:(h:15,i:'2ef1cf98-c298-441b-aa92-3dd8eb2a60f3',w:24,x:0,y:0),id:ed33b580-6d6b-11ec-bbd5-2b486ded5a8c,panelIndex:'2ef1cf98-c298-441b-aa92-3dd8eb2a60f3',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:c1e07834-d4ca-44fa-be7a-375ec76a2b79,w:24,x:24,y:0),id:'3d733a20-6d6c-11ec-bbd5-2b486ded5a8c',panelIndex:c1e07834-d4ca-44fa-be7a-375ec76a2b79,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:bc902b6e-3329-4242-8be6-2fd18edc2243,w:24,x:0,y:15),id:bc4ec7c0-6d7a-11ec-bbd5-2b486ded5a8c,panelIndex:bc902b6e-3329-4242-8be6-2fd18edc2243,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'8ef863ed-5672-4ae5-a31d-04a562aba2ea',w:24,x:24,y:15),id:'0bdb36c0-6d80-11ec-bbd5-2b486ded5a8c',panelIndex:'8ef863ed-5672-4ae5-a31d-04a562aba2ea',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'3fafc9c5-1f27-4464-a0f5-c839d6a5e6a4',w:24,x:0,y:30),id:'2dd74490-6d70-11ec-bbd5-2b486ded5a8c',panelIndex:'3fafc9c5-1f27-4464-a0f5-c839d6a5e6a4',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'2ca93202-aab7-468c-83ef-33122dd2e9be',w:24,x:24,y:30),id:'85f73c20-6d70-11ec-bbd5-2b486ded5a8c',panelIndex:'2ca93202-aab7-468c-83ef-33122dd2e9be',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'8db4ce85-167c-4f6c-9244-b6aaa102247c',w:24,x:0,y:45),id:'1195f010-6d6b-11ec-bbd5-2b486ded5a8c',panelIndex:'8db4ce85-167c-4f6c-9244-b6aaa102247c',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'45063606-812e-40ae-a2a5-7aaac6a30ff0',w:24,x:24,y:45),id:'825130d0-6d6b-11ec-bbd5-2b486ded5a8c',panelIndex:'45063606-812e-40ae-a2a5-7aaac6a30ff0',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'6ae09688-fa6e-4c1c-8520-b5fd2cc9495b',w:24,x:0,y:60),id:'7699bce0-6d79-11ec-bbd5-2b486ded5a8c',panelIndex:'6ae09688-fa6e-4c1c-8520-b5fd2cc9495b',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:e4fa0439-419e-44d1-bd03-58f52431f55c,w:24,x:24,y:60),id:d775e160-6d79-11ec-bbd5-2b486ded5a8c,panelIndex:e4fa0439-419e-44d1-bd03-58f52431f55c,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'1f66127b-f373-4977-a8ee-5359252232a4',w:24,x:0,y:75),id:d6fcbac0-6d6e-11ec-bbd5-2b486ded5a8c,panelIndex:'1f66127b-f373-4977-a8ee-5359252232a4',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'371f3c13-dda2-452c-b680-0e5ee85a2348',w:24,x:24,y:75),id:'2dd62890-6d6f-11ec-bbd5-2b486ded5a8c',panelIndex:'371f3c13-dda2-452c-b680-0e5ee85a2348',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'57a8b28e-a0aa-435f-a52c-8aae9595edc9',w:24,x:0,y:90),id:'5284a3d0-6d6d-11ec-bbd5-2b486ded5a8c',panelIndex:'57a8b28e-a0aa-435f-a52c-8aae9595edc9',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:f1c066f0-3f6b-41cc-b432-b32177791386,w:24,x:24,y:90),id:a609dd90-6d6d-11ec-bbd5-2b486ded5a8c,panelIndex:f1c066f0-3f6b-41cc-b432-b32177791386,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'17641913-3eb8-4c7d-973e-76cd73db0990',w:24,x:0,y:105),id:'48962150-6d81-11ec-bbd5-2b486ded5a8c',panelIndex:'17641913-3eb8-4c7d-973e-76cd73db0990',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:d4bd425c-604d-4236-ae17-1fd584bfc7a8,w:24,x:24,y:105),id:'2c32b720-6d82-11ec-bbd5-2b486ded5a8c',panelIndex:d4bd425c-604d-4236-ae17-1fd584bfc7a8,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'2b654ef0-d3d3-4c34-8101-3d0dc47ce343',w:24,x:0,y:120),id:'39b25600-6d76-11ec-bbd5-2b486ded5a8c',panelIndex:'2b654ef0-d3d3-4c34-8101-3d0dc47ce343',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:f58fdec4-9166-425c-9408-57e2f7730691,w:24,x:24,y:120),id:'95fc6f40-6d76-11ec-bbd5-2b486ded5a8c',panelIndex:f58fdec4-9166-425c-9408-57e2f7730691,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'356ffde0-779b-410b-a793-6efdbd243f83',w:24,x:0,y:135),id:'65b24340-6d77-11ec-bbd5-2b486ded5a8c',panelIndex:'356ffde0-779b-410b-a793-6efdbd243f83',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'568514b4-f4c0-4203-a2fd-c195c75531d9',w:24,x:24,y:135),id:'9f6893a0-6d68-11ec-bbd5-2b486ded5a8c',panelIndex:'568514b4-f4c0-4203-a2fd-c195c75531d9',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'1f4b9020-ed90-4d2d-92d8-84ca00aebc39',w:24,x:0,y:150),id:e7b66970-6d77-11ec-bbd5-2b486ded5a8c,panelIndex:'1f4b9020-ed90-4d2d-92d8-84ca00aebc39',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'2508db50-4016-4d2b-a58f-8e3c8c169965',w:24,x:24,y:150),id:'42afa110-6d6a-11ec-bbd5-2b486ded5a8c',panelIndex:'2508db50-4016-4d2b-a58f-8e3c8c169965',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'1899e965-f61b-467f-b76c-f2c737133deb',w:24,x:0,y:165),id:'5fedb600-6d78-11ec-bbd5-2b486ded5a8c',panelIndex:'1899e965-f61b-467f-b76c-f2c737133deb',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'93f0ecf7-4570-4627-82b9-afcd19510014',w:24,x:24,y:165),id:'980ca810-6d6a-11ec-bbd5-2b486ded5a8c',panelIndex:'93f0ecf7-4570-4627-82b9-afcd19510014',type:visualization,version:'7.16.2')),query:(language:kuery,query:''),tags:!(),timeRestore:!t,title:'Crowdsourced%20Measurements%20Dashboard%20(Min)',viewMode:view)";
        model.addAttribute("classActiveSettingsCrowdsourced", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchSubnetsMin";
    }
    
    @GetMapping(value = "/secure/crowdsourced/percentileValues")
    public String elasticsearchCrowdsourcedPercentile(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/3bdcaf40-6da6-11ec-bbd5-2b486ded5a8c?embed=true&_g=(filters:!())&_a=(description:'A%20dashboard%20related%20to%20the%2095th%20percentile%20values%20of%20Crowdsourced%20Measurements',filters:!(('$state':(store:appState),meta:(alias:!n,disabled:!f,index:wifimon_v0.1,key:Origin,negate:!f,params:(query:User),type:phrase),query:(match_phrase:(Origin:User)))),fullScreenMode:!f,options:(hidePanelTitles:!f,syncColors:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:()),gridData:(h:15,i:'78badee3-101c-4c44-bedf-61f28392c057',w:24,x:0,y:0),id:'09bfe7f0-6d6c-11ec-bbd5-2b486ded5a8c',panelIndex:'78badee3-101c-4c44-bedf-61f28392c057',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'54f4e0d1-0c92-4fe2-9708-d501e99d7010',w:24,x:24,y:0),id:'5bfc26f0-6d6c-11ec-bbd5-2b486ded5a8c',panelIndex:'54f4e0d1-0c92-4fe2-9708-d501e99d7010',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'133cb7b1-4e3a-457d-bb73-fc62bfc9ea90',w:24,x:0,y:15),id:f71d1a00-6d7a-11ec-bbd5-2b486ded5a8c,panelIndex:'133cb7b1-4e3a-457d-bb73-fc62bfc9ea90',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:df2e6490-4c2d-48c7-8f84-00cf643c0980,w:24,x:24,y:15),id:'24c4ee10-6d80-11ec-bbd5-2b486ded5a8c',panelIndex:df2e6490-4c2d-48c7-8f84-00cf643c0980,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:d3410a8c-5305-4a80-821f-fc9ca76bb36a,w:24,x:0,y:30),id:'4becfd30-6d70-11ec-bbd5-2b486ded5a8c',panelIndex:d3410a8c-5305-4a80-821f-fc9ca76bb36a,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'0ed3ebb1-f1a9-49f6-b05c-6c602bca3acf',w:24,x:24,y:30),id:d30be6a0-6d70-11ec-bbd5-2b486ded5a8c,panelIndex:'0ed3ebb1-f1a9-49f6-b05c-6c602bca3acf',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:a0182012-3a4a-476e-a41e-173482021a7a,w:24,x:0,y:45),id:'2e879b10-6d6b-11ec-bbd5-2b486ded5a8c',panelIndex:a0182012-3a4a-476e-a41e-173482021a7a,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'01e77322-7c77-4424-b113-46fb2b1a3844',w:24,x:24,y:45),id:'9c328e90-6d6b-11ec-bbd5-2b486ded5a8c',panelIndex:'01e77322-7c77-4424-b113-46fb2b1a3844',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'53aa1190-bde3-4d99-9e64-f6042bf37344',w:24,x:0,y:60),id:a84976e0-6d79-11ec-bbd5-2b486ded5a8c,panelIndex:'53aa1190-bde3-4d99-9e64-f6042bf37344',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:a00dc45b-4253-48a6-8424-110bb256f3e7,w:24,x:24,y:60),id:f46a2470-6d79-11ec-bbd5-2b486ded5a8c,panelIndex:a00dc45b-4253-48a6-8424-110bb256f3e7,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'76ea6b07-6524-4afb-b05a-a78e9fd2a667',w:24,x:0,y:75),id:f1cc7020-6d6e-11ec-bbd5-2b486ded5a8c,panelIndex:'76ea6b07-6524-4afb-b05a-a78e9fd2a667',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:bc928c11-978b-4127-a60d-da667b36cb73,w:24,x:24,y:75),id:'45138f70-6d6f-11ec-bbd5-2b486ded5a8c',panelIndex:bc928c11-978b-4127-a60d-da667b36cb73,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'4430b8ea-7740-4eda-8ffb-fe28e472f6b4',w:24,x:0,y:90),id:'826dcea0-6d6d-11ec-bbd5-2b486ded5a8c',panelIndex:'4430b8ea-7740-4eda-8ffb-fe28e472f6b4',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'67869f6f-0038-4c15-a010-8900f02ef2cb',w:24,x:24,y:90),id:bd6ad200-6d6d-11ec-bbd5-2b486ded5a8c,panelIndex:'67869f6f-0038-4c15-a010-8900f02ef2cb',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'9dedf5f1-e670-4dc9-bfe3-ca36928a547b',w:24,x:0,y:105),id:'5c706000-6d81-11ec-bbd5-2b486ded5a8c',panelIndex:'9dedf5f1-e670-4dc9-bfe3-ca36928a547b',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:b3396d7f-0097-4ecb-b0a6-6503536c3262,w:24,x:24,y:105),id:'457319f0-6d82-11ec-bbd5-2b486ded5a8c',panelIndex:b3396d7f-0097-4ecb-b0a6-6503536c3262,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'2191b665-2416-41c7-9c32-a83b7cfc6ff0',w:24,x:0,y:120),id:'5673a230-6d76-11ec-bbd5-2b486ded5a8c',panelIndex:'2191b665-2416-41c7-9c32-a83b7cfc6ff0',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:d0fc2f4a-6904-4fe6-ab74-5838f87560dd,w:24,x:24,y:120),id:bfefe020-6d76-11ec-bbd5-2b486ded5a8c,panelIndex:d0fc2f4a-6904-4fe6-ab74-5838f87560dd,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'2050b447-6e42-4e49-892c-0ee90c1362c3',w:24,x:0,y:135),id:af57deb0-6d77-11ec-bbd5-2b486ded5a8c,panelIndex:'2050b447-6e42-4e49-892c-0ee90c1362c3',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'1dad4acc-65fb-4dde-8395-d873e7407223',w:24,x:24,y:135),id:d0d1b240-6d69-11ec-bbd5-2b486ded5a8c,panelIndex:'1dad4acc-65fb-4dde-8395-d873e7407223',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:e655aa29-29b4-4804-893a-4f8843f827b7,w:24,x:0,y:150),id:'118d2a90-6d78-11ec-bbd5-2b486ded5a8c',panelIndex:e655aa29-29b4-4804-893a-4f8843f827b7,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'7d11e504-de4e-4549-975a-ac7b6e2dafb5',w:24,x:24,y:150),id:'5e47d690-6d6a-11ec-bbd5-2b486ded5a8c',panelIndex:'7d11e504-de4e-4549-975a-ac7b6e2dafb5',type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:cbcb3a25-7363-4e17-87fb-9e11a9a95d62,w:24,x:0,y:165),id:'8961e4c0-6d78-11ec-bbd5-2b486ded5a8c',panelIndex:cbcb3a25-7363-4e17-87fb-9e11a9a95d62,type:visualization,version:'7.16.2'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:b6c26e72-878a-4e4d-9cb1-93aff0576283,w:24,x:24,y:165),id:b5467fa0-6d6a-11ec-bbd5-2b486ded5a8c,panelIndex:b6c26e72-878a-4e4d-9cb1-93aff0576283,type:visualization,version:'7.16.2')),query:(language:kuery,query:''),tags:!(),timeRestore:!t,title:'Crowdsourced%20Measurements%20Dashboard%20(95th%20Percentile)',viewMode:view)";
        model.addAttribute("classActiveSettingsCrowdsourced", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchSubnetsPercentile";
    }

    @GetMapping(value = "/secure/hardwareProbes/averageValues")
    public String elasticsearchProbesAverage(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/e537f2f0-0b80-11ea-a8ab-cf1a4435820b?embed=true&_g=(filters:!())&_a=(description:'A%20dashboard%20related%20to%20average%20values%20of%20WiFiMon%20Hardware%20Probe%20Measurements',filters:!(('$state':(store:appState),meta:(alias:!n,disabled:!f,index:wifimon_v0.1,key:Origin,negate:!f,params:(query:Probe),type:phrase),query:(match_phrase:(Origin:Probe)))),fullScreenMode:!f,options:(darkTheme:!f,hidePanelTitles:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'1',w:24,x:0,y:0),id:'24fcad00-0b80-11ea-a8ab-cf1a4435820b',panelIndex:'1',title:'Average%20Download%20Throughput%20for%20WiFiMon%20Hardware%20Probes%20(per%20Test%20Tool)',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'2',w:24,x:24,y:0),id:'2c3d5d80-0b80-11ea-a8ab-cf1a4435820b',panelIndex:'2',title:'Average%20Download%20Throughput%20for%20WiFiMon%20Hardware%20Probes%20(Aggregated%20all%20Test%20Tools)',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'3',w:24,x:0,y:15),id:'44643a00-0b80-11ea-a8ab-cf1a4435820b',panelIndex:'3',title:'Average%20Upload%20Throughput%20for%20WiFiMon%20Hardware%20Probes%20(per%20Test%20Tool)',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'4',w:24,x:24,y:15),id:'4d2aa020-0b80-11ea-a8ab-cf1a4435820b',panelIndex:'4',title:'Average%20Upload%20Throughput%20for%20WiFiMon%20Hardware%20Probes%20(Aggregated%20all%20Test%20Tools)',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'6',w:24,x:24,y:30),id:'3c3558f0-0b80-11ea-a8ab-cf1a4435820b',panelIndex:'6',title:'Average%20HTTP%20Ping%20Round%20Trip%20Time%20for%20WiFiMon%20Hardware%20Probes%20(Aggregated%20all%20Test%20Tools)',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'7',w:24,x:0,y:60),id:'89692430-0b80-11ea-a8ab-cf1a4435820b',panelIndex:'7',title:'Average%20Bit%20Rate%20for%20WiFiMon%20Hardware%20Probes%20(per%20Probe)',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'8',w:24,x:24,y:60),id:a11f1670-0b80-11ea-a8ab-cf1a4435820b,panelIndex:'8',title:'Average%20Link%20Quality%20for%20WiFiMon%20Hardware%20Probes%20(per%20Probe)',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'9',w:24,x:0,y:75),id:afb956a0-0b80-11ea-a8ab-cf1a4435820b,panelIndex:'9',title:'Average%20Signal%20Level%20for%20WiFiMon%20Hardware%20Probes%20(per%20Probe)',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:'10',w:24,x:24,y:75),id:bd4bc3c0-0b80-11ea-a8ab-cf1a4435820b,panelIndex:'10',title:'Average%20TX%20Power%20for%20WiFiMon%20Hardware%20Probes%20(per%20Probe)',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:(),hidePanelTitles:!f),gridData:(h:15,i:db5049c8-207a-4eeb-a1b2-3d1303f1f5b6,w:24,x:0,y:30),id:fa05d330-145a-11eb-ad81-870789fb8a8b,panelIndex:db5049c8-207a-4eeb-a1b2-3d1303f1f5b6,title:'Average%20HTTP%20Ping%20Round%20Trip%20Time%20for%20WiFiMon%20Hardware%20Probes%20(per%20Test%20Tool)',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:c03bc990-d180-4f8c-bed1-88fa5e6bf637,w:24,x:0,y:45),id:'3aa48500-2de3-11ed-a51a-691288c59440',panelIndex:c03bc990-d180-4f8c-bed1-88fa5e6bf637,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'7d904341-951d-4a68-8467-d5eec982a38e',w:24,x:24,y:45),id:'9e8006d0-2de3-11ed-a51a-691288c59440',panelIndex:'7d904341-951d-4a68-8467-d5eec982a38e',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'4591ded1-b1e5-4795-8a54-bf7eb0517e0b',w:24,x:0,y:90),id:ad067b60-1d91-11ed-8cbc-af6abe82c7ab,panelIndex:'4591ded1-b1e5-4795-8a54-bf7eb0517e0b',type:visualization,version:'7.17.5'),(embeddableConfig:(attributes:(description:'A%20table%20describing%20the%20WiFiMon%20Hardware%20Probes%20used%20by%20the%20WiFiMon%20administrator%20and%20includes%20information%20about%20the%20associated%20access%20point%20and%20the%20location%20of%20the%20probe',references:!((id:probes_v0.1,name:indexpattern-datasource-current-indexpattern,type:index-pattern),(id:probes_v0.1,name:indexpattern-datasource-layer-57e9160b-ceb8-4209-94b1-4192fa3908fb,type:index-pattern)),state:(datasourceStates:(indexpattern:(layers:('57e9160b-ceb8-4209-94b1-4192fa3908fb':(columnOrder:!(f86edc2e-1373-4086-a632-8ea2d3173ae3,'7a116979-977c-4be0-847a-4696331503d7','0f20491f-2c72-4434-a9b7-3e3c1621d89e','816a60ad-dee8-40e0-897c-3fcce98fa5ba','3a943357-eb7c-4126-bd79-2f5d77819c3e','45ec9300-9aed-4d55-bb09-c93fb86708c5'),columns:('0f20491f-2c72-4434-a9b7-3e3c1621d89e':(customLabel:!t,dataType:string,isBucketed:!f,label:'Test%20Device%20Location',operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Test-Device-Location-Description.keyword),'3a943357-eb7c-4126-bd79-2f5d77819c3e':(customLabel:!t,dataType:number,isBucketed:!f,label:'Signal%20Level%20(Median)',operationType:median,scale:ratio,sourceField:Signal-Level),'45ec9300-9aed-4d55-bb09-c93fb86708c5':(customLabel:!t,dataType:number,isBucketed:!f,label:'Link-Quality%20(Median)',operationType:median,scale:ratio,sourceField:Link-Quality),'7a116979-977c-4be0-847a-4696331503d7':(customLabel:!t,dataType:string,isBucketed:!f,label:'Location%20Name',operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Location-Name.keyword),'816a60ad-dee8-40e0-897c-3fcce98fa5ba':(customLabel:!t,dataType:string,isBucketed:!f,label:Accesspoint,operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Accesspoint),f86edc2e-1373-4086-a632-8ea2d3173ae3:(customLabel:!t,dataType:string,isBucketed:!t,label:'Probe%20Number',operationType:terms,params:(missingBucket:!f,orderBy:(columnId:'45ec9300-9aed-4d55-bb09-c93fb86708c5',type:column),orderDirection:desc,otherBucket:!t,size:20),scale:ordinal,sourceField:Probe-No)),incompleteColumns:())))),filters:!(),query:(language:lucene,query:''),visualization:(columns:!((alignment:center,columnId:f86edc2e-1373-4086-a632-8ea2d3173ae3,isTransposed:!f),(alignment:center,columnId:'7a116979-977c-4be0-847a-4696331503d7',isTransposed:!f),(alignment:center,columnId:'0f20491f-2c72-4434-a9b7-3e3c1621d89e',isTransposed:!f),(alignment:center,columnId:'816a60ad-dee8-40e0-897c-3fcce98fa5ba',isTransposed:!f),(alignment:center,columnId:'3a943357-eb7c-4126-bd79-2f5d77819c3e',isTransposed:!f),(alignment:center,columnId:'45ec9300-9aed-4d55-bb09-c93fb86708c5',isTransposed:!f)),layerId:'57e9160b-ceb8-4209-94b1-4192fa3908fb',layerType:data)),title:'Probes%20Description%20Table',type:lens,visualizationType:lnsDatatable),enhancements:(),hidePanelTitles:!f),gridData:(h:12,i:'0e87bfc4-2be0-4361-9bf3-083447d686cb',w:48,x:0,y:105),panelIndex:'0e87bfc4-2be0-4361-9bf3-083447d686cb',title:'WiFiMon%20Hardware%20Probes%20Summary%20(Number,%20Location,%20etc.)',type:lens,version:'7.17.5')),query:(language:lucene,query:''),tags:!(),timeRestore:!t,title:'WiFiMon%20Hardware%20Probes%20(Average%20Values)',viewMode:view)";
        model.addAttribute("classActiveSettingsHardwareProbes", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchProbesAverage";
    }

    @GetMapping(value = "/secure/hardwareProbes/medianValues")
    public String elasticsearchProbesMedian(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/18b98780-3d9b-11ec-8eec-09df95559c81?embed=True&_g=(filters:!())&_a=(description:'A%20dashboard%20related%20to%20median%20values%20of%20WiFiMon%20Hardware%20Probe%20Measurements',filters:!(('$state':(store:appState),meta:(alias:!n,disabled:!f,index:wifimon_v0.1,key:Origin,negate:!f,params:(query:Probe),type:phrase),query:(match_phrase:(Origin:Probe)))),fullScreenMode:!f,options:(hidePanelTitles:!f,syncColors:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:()),gridData:(h:15,i:e2afc9f5-b5b4-4cce-9eda-0151a650b1ec,w:24,x:0,y:0),id:'41793450-3d90-11ec-8eec-09df95559c81',panelIndex:e2afc9f5-b5b4-4cce-9eda-0151a650b1ec,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:cef119d1-0c71-4c97-b024-8f819a5cb1bd,w:24,x:24,y:0),id:'492dbfd0-3d91-11ec-8eec-09df95559c81',panelIndex:cef119d1-0c71-4c97-b024-8f819a5cb1bd,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'84f4bcd5-86a3-4eba-8797-667e4e076dfb',w:24,x:0,y:15),id:e2b265d0-3d90-11ec-8eec-09df95559c81,panelIndex:'84f4bcd5-86a3-4eba-8797-667e4e076dfb',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'20faf6a7-2413-43f9-9d17-7f3361e36140',w:24,x:24,y:15),id:'8f18c440-3d91-11ec-8eec-09df95559c81',panelIndex:'20faf6a7-2413-43f9-9d17-7f3361e36140',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'80248c7b-9264-4501-aeb5-5267d6219fc9',w:24,x:0,y:30),id:b68e3240-3d90-11ec-8eec-09df95559c81,panelIndex:'80248c7b-9264-4501-aeb5-5267d6219fc9',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:e73e24a7-caca-4c4c-9d34-8b8c1c769a54,w:24,x:24,y:30),id:c95f48e0-3d91-11ec-8eec-09df95559c81,panelIndex:e73e24a7-caca-4c4c-9d34-8b8c1c769a54,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:fa37764c-9f62-4c0a-9483-16b650da03fb,w:24,x:0,y:45),id:'78ccbb30-2de4-11ed-a51a-691288c59440',panelIndex:fa37764c-9f62-4c0a-9483-16b650da03fb,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:ea64880d-a91b-4a9f-ab6a-5ae4dd9f22ce,w:24,x:24,y:60),id:'81e9de70-3d92-11ec-8eec-09df95559c81',panelIndex:ea64880d-a91b-4a9f-ab6a-5ae4dd9f22ce,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:b30202ec-9b04-4630-b21a-e23ccaaeeb35,w:24,x:0,y:60),id:fd9ba130-3d91-11ec-8eec-09df95559c81,panelIndex:b30202ec-9b04-4630-b21a-e23ccaaeeb35,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'912c8ea5-491d-48f9-9717-3e2185ba8d59',w:24,x:24,y:75),id:bd4bc3c0-0b80-11ea-a8ab-cf1a4435820b,panelIndex:'912c8ea5-491d-48f9-9717-3e2185ba8d59',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'5dd32b83-6b60-4572-a1e1-aec0a99e5b62',w:24,x:0,y:75),id:'2020c1e0-3d92-11ec-8eec-09df95559c81',panelIndex:'5dd32b83-6b60-4572-a1e1-aec0a99e5b62',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:bfd9e336-14d5-43a3-938c-fc1191b92b4e,w:24,x:24,y:45),id:bf19dcd0-2de4-11ed-a51a-691288c59440,panelIndex:bfd9e336-14d5-43a3-938c-fc1191b92b4e,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'5eac392a-e876-4576-9011-a627685aa28b',w:24,x:0,y:90),id:d7f9d010-1d91-11ed-8cbc-af6abe82c7ab,panelIndex:'5eac392a-e876-4576-9011-a627685aa28b',type:visualization,version:'7.17.5'),(embeddableConfig:(attributes:(description:'A%20table%20describing%20the%20WiFiMon%20Hardware%20Probes%20used%20by%20the%20WiFiMon%20administrator%20and%20includes%20information%20about%20the%20associated%20access%20point%20and%20the%20location%20of%20the%20probe',references:!((id:probes_v0.1,name:indexpattern-datasource-current-indexpattern,type:index-pattern),(id:probes_v0.1,name:indexpattern-datasource-layer-57e9160b-ceb8-4209-94b1-4192fa3908fb,type:index-pattern)),state:(datasourceStates:(indexpattern:(layers:('57e9160b-ceb8-4209-94b1-4192fa3908fb':(columnOrder:!(f86edc2e-1373-4086-a632-8ea2d3173ae3,'7a116979-977c-4be0-847a-4696331503d7','0f20491f-2c72-4434-a9b7-3e3c1621d89e','816a60ad-dee8-40e0-897c-3fcce98fa5ba','3a943357-eb7c-4126-bd79-2f5d77819c3e','45ec9300-9aed-4d55-bb09-c93fb86708c5'),columns:('0f20491f-2c72-4434-a9b7-3e3c1621d89e':(customLabel:!t,dataType:string,isBucketed:!f,label:'Test%20Device%20Location',operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Test-Device-Location-Description.keyword),'3a943357-eb7c-4126-bd79-2f5d77819c3e':(customLabel:!t,dataType:number,isBucketed:!f,label:'Signal%20Level%20(Median)',operationType:median,scale:ratio,sourceField:Signal-Level),'45ec9300-9aed-4d55-bb09-c93fb86708c5':(customLabel:!t,dataType:number,isBucketed:!f,label:'Link-Quality%20(Median)',operationType:median,scale:ratio,sourceField:Link-Quality),'7a116979-977c-4be0-847a-4696331503d7':(customLabel:!t,dataType:string,isBucketed:!f,label:'Location%20Name',operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Location-Name.keyword),'816a60ad-dee8-40e0-897c-3fcce98fa5ba':(customLabel:!t,dataType:string,isBucketed:!f,label:Accesspoint,operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Accesspoint),f86edc2e-1373-4086-a632-8ea2d3173ae3:(customLabel:!t,dataType:string,isBucketed:!t,label:'Probe%20Number',operationType:terms,params:(missingBucket:!f,orderBy:(columnId:'45ec9300-9aed-4d55-bb09-c93fb86708c5',type:column),orderDirection:desc,otherBucket:!t,size:20),scale:ordinal,sourceField:Probe-No)),incompleteColumns:())))),filters:!(),query:(language:lucene,query:''),visualization:(columns:!((alignment:center,columnId:f86edc2e-1373-4086-a632-8ea2d3173ae3,isTransposed:!f),(alignment:center,columnId:'7a116979-977c-4be0-847a-4696331503d7',isTransposed:!f),(alignment:center,columnId:'0f20491f-2c72-4434-a9b7-3e3c1621d89e',isTransposed:!f),(alignment:center,columnId:'816a60ad-dee8-40e0-897c-3fcce98fa5ba',isTransposed:!f),(alignment:center,columnId:'3a943357-eb7c-4126-bd79-2f5d77819c3e',isTransposed:!f),(alignment:center,columnId:'45ec9300-9aed-4d55-bb09-c93fb86708c5',isTransposed:!f)),layerId:'57e9160b-ceb8-4209-94b1-4192fa3908fb',layerType:data)),title:'Probes%20Description%20Table',type:lens,visualizationType:lnsDatatable),enhancements:(),hidePanelTitles:!f),gridData:(h:12,i:'2c6403f1-b119-4ce6-a388-0ecf0a0206b7',w:48,x:0,y:105),panelIndex:'2c6403f1-b119-4ce6-a388-0ecf0a0206b7',title:'WiFiMon%20Hardware%20Probes%20Summary%20(Number,%20Location,%20etc.)',type:lens,version:'7.17.5')),query:(language:kuery,query:''),tags:!(),timeRestore:!t,title:'WiFiMon%20Hardware%20Probes%20(Median%20Values)',viewMode:view)";
        model.addAttribute("classActiveSettingsHardwareProbes", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchProbesMedian";
    }
    
    @GetMapping(value = "/secure/hardwareProbes/maxValues")
    public String elasticsearchProbesMax(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/a5072e40-6da7-11ec-bbd5-2b486ded5a8c?embed=true&_g=(filters:!())&_a=(description:'A%20dashboard%20related%20to%20max%20values%20of%20WiFiMon%20Hardware%20Probe%20Measurements',filters:!(('$state':(store:appState),meta:(alias:!n,disabled:!f,index:wifimon_v0.1,key:Origin,negate:!f,params:(query:Probe),type:phrase),query:(match_phrase:(Origin:Probe)))),fullScreenMode:!f,options:(hidePanelTitles:!f,syncColors:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:()),gridData:(h:15,i:dc26d094-c1d8-45f8-ba66-2a75957180a7,w:24,x:0,y:0),id:'636767e0-6d82-11ec-bbd5-2b486ded5a8c',panelIndex:dc26d094-c1d8-45f8-ba66-2a75957180a7,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'0d904e2c-bdb3-4d9d-b01d-9d7f6774e29b',w:24,x:24,y:0),id:a64b1fc0-6d82-11ec-bbd5-2b486ded5a8c,panelIndex:'0d904e2c-bdb3-4d9d-b01d-9d7f6774e29b',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'19546af3-0ae9-4c85-9ed6-adacaf5e086a',w:24,x:0,y:15),id:aad916e0-6d83-11ec-bbd5-2b486ded5a8c,panelIndex:'19546af3-0ae9-4c85-9ed6-adacaf5e086a',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'267f6ec4-0316-41a2-8bc1-7c03e1ddb6e3',w:24,x:24,y:15),id:e450d9d0-6d83-11ec-bbd5-2b486ded5a8c,panelIndex:'267f6ec4-0316-41a2-8bc1-7c03e1ddb6e3',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'4feed6a3-7576-41b6-b378-95f514e67092',w:24,x:0,y:30),id:'03d4bde0-6d83-11ec-bbd5-2b486ded5a8c',panelIndex:'4feed6a3-7576-41b6-b378-95f514e67092',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'96b42c4d-6dbd-4a74-a2e8-bbb3f3cc104e',w:24,x:24,y:30),id:'50a3d3e0-6d83-11ec-bbd5-2b486ded5a8c',panelIndex:'96b42c4d-6dbd-4a74-a2e8-bbb3f3cc104e',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'9cbc8b43-a5e4-4238-a449-9b5b0ce70d67',w:24,x:0,y:45),id:f2034f50-2de4-11ed-a51a-691288c59440,panelIndex:'9cbc8b43-a5e4-4238-a449-9b5b0ce70d67',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'80393d15-a74c-44a9-8c95-a5a1ffa70653',w:24,x:24,y:60),id:'4aa8d380-6d67-11ec-bbd5-2b486ded5a8c',panelIndex:'80393d15-a74c-44a9-8c95-a5a1ffa70653',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'0a009b33-a9fc-4e6d-8281-936ce5a00f3d',w:24,x:0,y:60),id:ecb66170-6d66-11ec-bbd5-2b486ded5a8c,panelIndex:'0a009b33-a9fc-4e6d-8281-936ce5a00f3d',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'7e931a26-e74a-4c88-9e59-f5cff8cce31c',w:24,x:24,y:75),id:da1b6410-6d67-11ec-bbd5-2b486ded5a8c,panelIndex:'7e931a26-e74a-4c88-9e59-f5cff8cce31c',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'0b4335e0-387b-4f2b-938d-8bcaf34b61c3',w:24,x:0,y:75),id:'93dc7340-6d67-11ec-bbd5-2b486ded5a8c',panelIndex:'0b4335e0-387b-4f2b-938d-8bcaf34b61c3',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'762d1b3d-ba02-4e91-8377-6319d35c7251',w:24,x:24,y:45),id:'1ad04190-2de5-11ed-a51a-691288c59440',panelIndex:'762d1b3d-ba02-4e91-8377-6319d35c7251',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'5a692e31-88c8-4c36-b633-b8f34d63840f',w:24,x:0,y:90),id:f4219b60-1d91-11ed-8cbc-af6abe82c7ab,panelIndex:'5a692e31-88c8-4c36-b633-b8f34d63840f',type:visualization,version:'7.17.5'),(embeddableConfig:(attributes:(description:'A%20table%20describing%20the%20WiFiMon%20Hardware%20Probes%20used%20by%20the%20WiFiMon%20administrator%20and%20includes%20information%20about%20the%20associated%20access%20point%20and%20the%20location%20of%20the%20probe',references:!((id:probes_v0.1,name:indexpattern-datasource-current-indexpattern,type:index-pattern),(id:probes_v0.1,name:indexpattern-datasource-layer-57e9160b-ceb8-4209-94b1-4192fa3908fb,type:index-pattern)),state:(datasourceStates:(indexpattern:(layers:('57e9160b-ceb8-4209-94b1-4192fa3908fb':(columnOrder:!(f86edc2e-1373-4086-a632-8ea2d3173ae3,'7a116979-977c-4be0-847a-4696331503d7','0f20491f-2c72-4434-a9b7-3e3c1621d89e','816a60ad-dee8-40e0-897c-3fcce98fa5ba','3a943357-eb7c-4126-bd79-2f5d77819c3e','45ec9300-9aed-4d55-bb09-c93fb86708c5'),columns:('0f20491f-2c72-4434-a9b7-3e3c1621d89e':(customLabel:!t,dataType:string,isBucketed:!f,label:'Test%20Device%20Location',operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Test-Device-Location-Description.keyword),'3a943357-eb7c-4126-bd79-2f5d77819c3e':(customLabel:!t,dataType:number,isBucketed:!f,label:'Signal%20Level%20(Median)',operationType:median,scale:ratio,sourceField:Signal-Level),'45ec9300-9aed-4d55-bb09-c93fb86708c5':(customLabel:!t,dataType:number,isBucketed:!f,label:'Link-Quality%20(Median)',operationType:median,scale:ratio,sourceField:Link-Quality),'7a116979-977c-4be0-847a-4696331503d7':(customLabel:!t,dataType:string,isBucketed:!f,label:'Location%20Name',operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Location-Name.keyword),'816a60ad-dee8-40e0-897c-3fcce98fa5ba':(customLabel:!t,dataType:string,isBucketed:!f,label:Accesspoint,operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Accesspoint),f86edc2e-1373-4086-a632-8ea2d3173ae3:(customLabel:!t,dataType:string,isBucketed:!t,label:'Probe%20Number',operationType:terms,params:(missingBucket:!f,orderBy:(columnId:'45ec9300-9aed-4d55-bb09-c93fb86708c5',type:column),orderDirection:desc,otherBucket:!t,size:20),scale:ordinal,sourceField:Probe-No)),incompleteColumns:())))),filters:!(),query:(language:lucene,query:''),visualization:(columns:!((alignment:center,columnId:f86edc2e-1373-4086-a632-8ea2d3173ae3,isTransposed:!f),(alignment:center,columnId:'7a116979-977c-4be0-847a-4696331503d7',isTransposed:!f),(alignment:center,columnId:'0f20491f-2c72-4434-a9b7-3e3c1621d89e',isTransposed:!f),(alignment:center,columnId:'816a60ad-dee8-40e0-897c-3fcce98fa5ba',isTransposed:!f),(alignment:center,columnId:'3a943357-eb7c-4126-bd79-2f5d77819c3e',isTransposed:!f),(alignment:center,columnId:'45ec9300-9aed-4d55-bb09-c93fb86708c5',isTransposed:!f)),layerId:'57e9160b-ceb8-4209-94b1-4192fa3908fb',layerType:data)),title:'Probes%20Description%20Table',type:lens,visualizationType:lnsDatatable),enhancements:(),hidePanelTitles:!f),gridData:(h:14,i:ab7d3949-7b92-4c03-90fd-9397451aa63c,w:48,x:0,y:105),panelIndex:ab7d3949-7b92-4c03-90fd-9397451aa63c,title:'WiFiMon%20Hardware%20Probes%20Summary%20(Number,%20Location,%20etc.)',type:lens,version:'7.17.5')),query:(language:kuery,query:''),tags:!(),timeRestore:!t,title:'WiFiMon%20Hardware%20Probes%20(Max%20Values)',viewMode:view)";
        model.addAttribute("classActiveSettingsHardwareProbes", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchProbesMax";
    }
    
    @GetMapping(value = "/secure/hardwareProbes/minValues")
    public String elasticsearchProbesMin(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/a6377410-6dab-11ec-bbd5-2b486ded5a8c?embed=true&_g=(filters:!())&_a=(description:'A%20dashboard%20related%20to%20average%20values%20of%20WiFiMon%20Hardware%20Probe%20Measurements',filters:!(('$state':(store:appState),meta:(alias:!n,disabled:!f,index:wifimon_v0.1,key:Origin,negate:!f,params:(query:Probe),type:phrase),query:(match_phrase:(Origin:Probe)))),fullScreenMode:!f,options:(hidePanelTitles:!f,syncColors:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:()),gridData:(h:15,i:c1e9852c-56f7-419e-81ca-7a587abb8d35,w:24,x:0,y:0),id:'73947d10-6d82-11ec-bbd5-2b486ded5a8c',panelIndex:c1e9852c-56f7-419e-81ca-7a587abb8d35,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'496b2e05-6665-429b-b74a-fc0f82d881cb',w:24,x:24,y:0),id:b870a990-6d82-11ec-bbd5-2b486ded5a8c,panelIndex:'496b2e05-6665-429b-b74a-fc0f82d881cb',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'7c1731d8-ee9e-4aa5-aebe-dbfa714266b0',w:24,x:0,y:15),id:bc28da20-6d83-11ec-bbd5-2b486ded5a8c,panelIndex:'7c1731d8-ee9e-4aa5-aebe-dbfa714266b0',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:d655261b-8dea-46ff-ae68-dafd51516680,w:24,x:24,y:15),id:f1285b60-6d83-11ec-bbd5-2b486ded5a8c,panelIndex:d655261b-8dea-46ff-ae68-dafd51516680,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:e027cc0b-bdba-42f5-a98c-9f21e282448f,w:24,x:0,y:30),id:'12664b80-6d83-11ec-bbd5-2b486ded5a8c',panelIndex:e027cc0b-bdba-42f5-a98c-9f21e282448f,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'043e7b65-1153-4185-921a-524e17e7646c',w:24,x:24,y:30),id:'6de01c70-6d83-11ec-bbd5-2b486ded5a8c',panelIndex:'043e7b65-1153-4185-921a-524e17e7646c',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'0bdd3df4-dc22-4e10-9f13-38b866b58c95',w:24,x:0,y:45),id:'3f2ba8e0-2de5-11ed-a51a-691288c59440',panelIndex:'0bdd3df4-dc22-4e10-9f13-38b866b58c95',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'5c7491d6-dd76-4acc-be04-2784528da341',w:24,x:24,y:60),id:'5a61a310-6d67-11ec-bbd5-2b486ded5a8c',panelIndex:'5c7491d6-dd76-4acc-be04-2784528da341',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'9f6c527d-de08-437d-a23d-57564e65c0ab',w:24,x:0,y:60),id:'0122a6f0-6d67-11ec-bbd5-2b486ded5a8c',panelIndex:'9f6c527d-de08-437d-a23d-57564e65c0ab',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'103c4175-6062-4072-b370-ef30cc218f29',w:24,x:24,y:75),id:e66b8d30-6d67-11ec-bbd5-2b486ded5a8c,panelIndex:'103c4175-6062-4072-b370-ef30cc218f29',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'5f043f60-2de5-4455-a7b0-7525b8c4bd46',w:24,x:0,y:75),id:a3ed26d0-6d67-11ec-bbd5-2b486ded5a8c,panelIndex:'5f043f60-2de5-4455-a7b0-7525b8c4bd46',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:d7752c5d-3c8c-46fe-9da0-3396ecf69fa3,w:24,x:24,y:45),id:'63fcb560-2de5-11ed-a51a-691288c59440',panelIndex:d7752c5d-3c8c-46fe-9da0-3396ecf69fa3,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:b3ec5ee8-33d1-4cd6-adba-6d13d1286594,w:24,x:0,y:90),id:'0c925220-1d92-11ed-8cbc-af6abe82c7ab',panelIndex:b3ec5ee8-33d1-4cd6-adba-6d13d1286594,type:visualization,version:'7.17.5'),(embeddableConfig:(attributes:(description:'A%20table%20describing%20the%20WiFiMon%20Hardware%20Probes%20used%20by%20the%20WiFiMon%20administrator%20and%20includes%20information%20about%20the%20associated%20access%20point%20and%20the%20location%20of%20the%20probe',references:!((id:probes_v0.1,name:indexpattern-datasource-current-indexpattern,type:index-pattern),(id:probes_v0.1,name:indexpattern-datasource-layer-57e9160b-ceb8-4209-94b1-4192fa3908fb,type:index-pattern)),state:(datasourceStates:(indexpattern:(layers:('57e9160b-ceb8-4209-94b1-4192fa3908fb':(columnOrder:!(f86edc2e-1373-4086-a632-8ea2d3173ae3,'7a116979-977c-4be0-847a-4696331503d7','0f20491f-2c72-4434-a9b7-3e3c1621d89e','816a60ad-dee8-40e0-897c-3fcce98fa5ba','3a943357-eb7c-4126-bd79-2f5d77819c3e','45ec9300-9aed-4d55-bb09-c93fb86708c5'),columns:('0f20491f-2c72-4434-a9b7-3e3c1621d89e':(customLabel:!t,dataType:string,isBucketed:!f,label:'Test%20Device%20Location',operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Test-Device-Location-Description.keyword),'3a943357-eb7c-4126-bd79-2f5d77819c3e':(customLabel:!t,dataType:number,isBucketed:!f,label:'Signal%20Level%20(Median)',operationType:median,scale:ratio,sourceField:Signal-Level),'45ec9300-9aed-4d55-bb09-c93fb86708c5':(customLabel:!t,dataType:number,isBucketed:!f,label:'Link-Quality%20(Median)',operationType:median,scale:ratio,sourceField:Link-Quality),'7a116979-977c-4be0-847a-4696331503d7':(customLabel:!t,dataType:string,isBucketed:!f,label:'Location%20Name',operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Location-Name.keyword),'816a60ad-dee8-40e0-897c-3fcce98fa5ba':(customLabel:!t,dataType:string,isBucketed:!f,label:Accesspoint,operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Accesspoint),f86edc2e-1373-4086-a632-8ea2d3173ae3:(customLabel:!t,dataType:string,isBucketed:!t,label:'Probe%20Number',operationType:terms,params:(missingBucket:!f,orderBy:(columnId:'45ec9300-9aed-4d55-bb09-c93fb86708c5',type:column),orderDirection:desc,otherBucket:!t,size:20),scale:ordinal,sourceField:Probe-No)),incompleteColumns:())))),filters:!(),query:(language:lucene,query:''),visualization:(columns:!((alignment:center,columnId:f86edc2e-1373-4086-a632-8ea2d3173ae3,isTransposed:!f),(alignment:center,columnId:'7a116979-977c-4be0-847a-4696331503d7',isTransposed:!f),(alignment:center,columnId:'0f20491f-2c72-4434-a9b7-3e3c1621d89e',isTransposed:!f),(alignment:center,columnId:'816a60ad-dee8-40e0-897c-3fcce98fa5ba',isTransposed:!f),(alignment:center,columnId:'3a943357-eb7c-4126-bd79-2f5d77819c3e',isTransposed:!f),(alignment:center,columnId:'45ec9300-9aed-4d55-bb09-c93fb86708c5',isTransposed:!f)),layerId:'57e9160b-ceb8-4209-94b1-4192fa3908fb',layerType:data)),title:'Probes%20Description%20Table',type:lens,visualizationType:lnsDatatable),enhancements:(),hidePanelTitles:!f),gridData:(h:14,i:'9f836cbc-0216-4241-b642-7d42a3c1009c',w:48,x:0,y:105),panelIndex:'9f836cbc-0216-4241-b642-7d42a3c1009c',title:'WiFiMon%20Hardware%20Probes%20Summary%20(Number,%20Location,%20etc.)',type:lens,version:'7.17.5')),query:(language:kuery,query:''),tags:!(),timeRestore:!t,title:'WiFiMon%20Hardware%20Probes%20(Min%20Values)',viewMode:view)";
        model.addAttribute("classActiveSettingsHardwareProbes", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchProbesMin";
    }
    
    @GetMapping(value = "/secure/hardwareProbes/percentileValues")
    public String elasticsearchProbesPercentile(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/ef7c2ad0-6dab-11ec-bbd5-2b486ded5a8c?embed=true&_g=(filters:!())&_a=(description:'A%20dashboard%20related%20to%20the%2095th%20percentile%20values%20of%20WiFiMon%20Hardware%20Probe%20Measurements',filters:!(('$state':(store:appState),meta:(alias:!n,disabled:!f,index:wifimon_v0.1,key:Origin,negate:!f,params:(query:Probe),type:phrase),query:(match_phrase:(Origin:Probe)))),fullScreenMode:!f,options:(hidePanelTitles:!f,syncColors:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:()),gridData:(h:15,i:'0d540033-149e-46c8-ba5e-ffb333aa512c',w:24,x:0,y:0),id:'8c4c27e0-6d82-11ec-bbd5-2b486ded5a8c',panelIndex:'0d540033-149e-46c8-ba5e-ffb333aa512c',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:aaf85de8-7336-4920-98f7-d4b41735c169,w:24,x:24,y:0),id:cf4dc800-6d82-11ec-bbd5-2b486ded5a8c,panelIndex:aaf85de8-7336-4920-98f7-d4b41735c169,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'68394ed5-2f33-408d-b164-5242a5130a71',w:24,x:0,y:15),id:d041cf80-6d83-11ec-bbd5-2b486ded5a8c,panelIndex:'68394ed5-2f33-408d-b164-5242a5130a71',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:bc00ff35-6485-4568-956e-fbc38c7a2ff0,w:24,x:24,y:15),id:'0776a750-6d84-11ec-bbd5-2b486ded5a8c',panelIndex:bc00ff35-6485-4568-956e-fbc38c7a2ff0,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'847c18d4-ca45-4a34-aa71-8445df7dfe31',w:24,x:0,y:30),id:'27da6410-6d83-11ec-bbd5-2b486ded5a8c',panelIndex:'847c18d4-ca45-4a34-aa71-8445df7dfe31',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'32187c36-e363-41e9-b1a6-291171d564b0',w:24,x:24,y:30),id:'89bed0d0-6d83-11ec-bbd5-2b486ded5a8c',panelIndex:'32187c36-e363-41e9-b1a6-291171d564b0',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'22d50f4b-4a82-4eed-90a7-a84a95db94ec',w:24,x:0,y:45),id:'9410e7d0-2de5-11ed-a51a-691288c59440',panelIndex:'22d50f4b-4a82-4eed-90a7-a84a95db94ec',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'09e15e0e-c447-4e34-b316-003dcb09f0c7',w:24,x:24,y:60),id:'76008f50-6d67-11ec-bbd5-2b486ded5a8c',panelIndex:'09e15e0e-c447-4e34-b316-003dcb09f0c7',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'7377fc99-3ad0-4e96-904e-2646e0317681',w:24,x:0,y:60),id:'25af7e80-6d67-11ec-bbd5-2b486ded5a8c',panelIndex:'7377fc99-3ad0-4e96-904e-2646e0317681',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'641a3d76-3663-4023-821c-ff2b9eb0b78a',w:24,x:24,y:75),id:'0c368a10-6d68-11ec-bbd5-2b486ded5a8c',panelIndex:'641a3d76-3663-4023-821c-ff2b9eb0b78a',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'7614a1f9-dccf-44a5-9ea9-55dab44cbbb3',w:24,x:0,y:75),id:bad47e70-6d67-11ec-bbd5-2b486ded5a8c,panelIndex:'7614a1f9-dccf-44a5-9ea9-55dab44cbbb3',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:ef35fcee-03de-4e58-88cf-c68f6cea354c,w:24,x:24,y:45),id:c3eddda0-2de5-11ed-a51a-691288c59440,panelIndex:ef35fcee-03de-4e58-88cf-c68f6cea354c,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'6f48e2a3-c4f5-4de5-84dd-738d48ac5739',w:24,x:0,y:90),id:'2ecd9890-1d92-11ed-8cbc-af6abe82c7ab',panelIndex:'6f48e2a3-c4f5-4de5-84dd-738d48ac5739',type:visualization,version:'7.17.5'),(embeddableConfig:(attributes:(description:'A%20table%20describing%20the%20WiFiMon%20Hardware%20Probes%20used%20by%20the%20WiFiMon%20administrator%20and%20includes%20information%20about%20the%20associated%20access%20point%20and%20the%20location%20of%20the%20probe',references:!((id:probes_v0.1,name:indexpattern-datasource-current-indexpattern,type:index-pattern),(id:probes_v0.1,name:indexpattern-datasource-layer-57e9160b-ceb8-4209-94b1-4192fa3908fb,type:index-pattern)),state:(datasourceStates:(indexpattern:(layers:('57e9160b-ceb8-4209-94b1-4192fa3908fb':(columnOrder:!(f86edc2e-1373-4086-a632-8ea2d3173ae3,'7a116979-977c-4be0-847a-4696331503d7','0f20491f-2c72-4434-a9b7-3e3c1621d89e','816a60ad-dee8-40e0-897c-3fcce98fa5ba','3a943357-eb7c-4126-bd79-2f5d77819c3e','45ec9300-9aed-4d55-bb09-c93fb86708c5'),columns:('0f20491f-2c72-4434-a9b7-3e3c1621d89e':(customLabel:!t,dataType:string,isBucketed:!f,label:'Test%20Device%20Location',operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Test-Device-Location-Description.keyword),'3a943357-eb7c-4126-bd79-2f5d77819c3e':(customLabel:!t,dataType:number,isBucketed:!f,label:'Signal%20Level%20(Median)',operationType:median,scale:ratio,sourceField:Signal-Level),'45ec9300-9aed-4d55-bb09-c93fb86708c5':(customLabel:!t,dataType:number,isBucketed:!f,label:'Link-Quality%20(Median)',operationType:median,scale:ratio,sourceField:Link-Quality),'7a116979-977c-4be0-847a-4696331503d7':(customLabel:!t,dataType:string,isBucketed:!f,label:'Location%20Name',operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Location-Name.keyword),'816a60ad-dee8-40e0-897c-3fcce98fa5ba':(customLabel:!t,dataType:string,isBucketed:!f,label:Accesspoint,operationType:last_value,params:(sortField:Timestamp),scale:ordinal,sourceField:Accesspoint),f86edc2e-1373-4086-a632-8ea2d3173ae3:(customLabel:!t,dataType:string,isBucketed:!t,label:'Probe%20Number',operationType:terms,params:(missingBucket:!f,orderBy:(columnId:'45ec9300-9aed-4d55-bb09-c93fb86708c5',type:column),orderDirection:desc,otherBucket:!t,size:20),scale:ordinal,sourceField:Probe-No)),incompleteColumns:())))),filters:!(),query:(language:lucene,query:''),visualization:(columns:!((alignment:center,columnId:f86edc2e-1373-4086-a632-8ea2d3173ae3,isTransposed:!f),(alignment:center,columnId:'7a116979-977c-4be0-847a-4696331503d7',isTransposed:!f),(alignment:center,columnId:'0f20491f-2c72-4434-a9b7-3e3c1621d89e',isTransposed:!f),(alignment:center,columnId:'816a60ad-dee8-40e0-897c-3fcce98fa5ba',isTransposed:!f),(alignment:center,columnId:'3a943357-eb7c-4126-bd79-2f5d77819c3e',isTransposed:!f),(alignment:center,columnId:'45ec9300-9aed-4d55-bb09-c93fb86708c5',isTransposed:!f)),layerId:'57e9160b-ceb8-4209-94b1-4192fa3908fb',layerType:data)),title:'Probes%20Description%20Table',type:lens,visualizationType:lnsDatatable),enhancements:(),hidePanelTitles:!f),gridData:(h:12,i:'79a0e945-e31e-44ac-a09c-1afeb6d5a0ad',w:48,x:0,y:105),panelIndex:'79a0e945-e31e-44ac-a09c-1afeb6d5a0ad',title:'WiFiMon%20Hardware%20Probes%20Summary%20(Number,%20Location,%20etc.)',type:lens,version:'7.17.5')),query:(language:kuery,query:''),tags:!(),timeRestore:!t,title:'WiFiMon%20Hardware%20Probes%20(95th%20Percentile)',viewMode:view)";
        model.addAttribute("classActiveSettingsHardwareProbes", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchProbesPercentile";
    }

    @GetMapping(value = "/secure/hardwareProbes/pingResults")
    public String elasticsearchPingResults(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/4136f910-3834-11ed-a51a-691288c59440?embed=true&_g=(filters:!())&_a=(description:'A%20dashboard%20including%20measurements%20from%20parsing%20the%20output%20of%20ping%20command',filters:!(),fullScreenMode:!f,options:(hidePanelTitles:!f,syncColors:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:()),gridData:(h:15,i:'6ebc5184-8054-431a-8b4e-1ac9a0c85388',w:24,x:0,y:0),id:'5a2dec40-3833-11ed-a51a-691288c59440',panelIndex:'6ebc5184-8054-431a-8b4e-1ac9a0c85388',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:a3f1a7a9-85e6-4b28-9828-0e25b6d6597e,w:24,x:24,y:0),id:'89bff660-3833-11ed-a51a-691288c59440',panelIndex:a3f1a7a9-85e6-4b28-9828-0e25b6d6597e,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'8b53b332-d42d-478e-ab54-e35288956eca',w:24,x:0,y:15),id:'6528be90-3833-11ed-a51a-691288c59440',panelIndex:'8b53b332-d42d-478e-ab54-e35288956eca',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'4461f3d5-77ca-44c4-ade6-62f7ed2fb577',w:24,x:24,y:15),id:'74bec3e0-3833-11ed-a51a-691288c59440',panelIndex:'4461f3d5-77ca-44c4-ade6-62f7ed2fb577',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'9698cae7-141f-4b5d-a681-212a71796bf3',w:24,x:0,y:30),id:'35fca230-3833-11ed-a51a-691288c59440',panelIndex:'9698cae7-141f-4b5d-a681-212a71796bf3',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'44c203b2-9f8e-4cf6-b3d0-cff01d99d11e',w:24,x:24,y:30),id:'48a3df20-3833-11ed-a51a-691288c59440',panelIndex:'44c203b2-9f8e-4cf6-b3d0-cff01d99d11e',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'1e60f197-8f83-4df8-aec3-f17570efceed',w:24,x:0,y:45),id:f7ab1ca0-3832-11ed-a51a-691288c59440,panelIndex:'1e60f197-8f83-4df8-aec3-f17570efceed',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'296ee340-23b0-4d10-8a84-3ccb1343ac1a',w:24,x:24,y:45),id:'01add350-3833-11ed-a51a-691288c59440',panelIndex:'296ee340-23b0-4d10-8a84-3ccb1343ac1a',type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:be07e566-41f9-480e-9e8c-74fa5268376b,w:24,x:0,y:60),id:'4c64fb40-3832-11ed-a51a-691288c59440',panelIndex:be07e566-41f9-480e-9e8c-74fa5268376b,type:visualization,version:'7.17.5'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:f1380588-a53b-4657-abb6-83b6235f5bbe,w:24,x:24,y:60),id:defee380-3832-11ed-a51a-691288c59440,panelIndex:f1380588-a53b-4657-abb6-83b6235f5bbe,type:visualization,version:'7.17.5')),query:(language:kuery,query:''),tags:!(),timeRestore:!t,title:'Dashboard%20for%20Ping%20Parser%20Measurements',viewMode:view)";
        model.addAttribute("classActiveSettingsProbes", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchProbesPingResults";
    }

    @GetMapping(value = "/secure/twamp/averageValues")
    public String elasticsearchTwampAverage(Model model, HttpSession session, HttpServletRequest request) {
        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
		"/app/dashboards#/view/de1abab0-aabc-11ec-a3d2-bd8ccacbecdb?embed=true&_g=(filters:!())&_a=(description:'A%20description%20for%20the%20average%20values%20of%20measurements%20collected%20by%20TWAMP',filters:!(),fullScreenMode:!f,options:(hidePanelTitles:!f,syncColors:!f,useMargins:!t),panels:!((embeddableConfig:(enhancements:()),gridData:(h:15,i:dbb03965-c8b1-43bc-9ab5-82fd061f3c05,w:24,x:0,y:0),id:'70fd9a30-aaba-11ec-a3d2-bd8ccacbecdb',panelIndex:dbb03965-c8b1-43bc-9ab5-82fd061f3c05,type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'448a8088-8708-4ce3-b119-a5ac1aceee9e',w:24,x:24,y:0),id:d2fb3440-aaba-11ec-a3d2-bd8ccacbecdb,panelIndex:'448a8088-8708-4ce3-b119-a5ac1aceee9e',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:b43e0a6e-2694-4cbe-a364-523cbddc1382,w:24,x:0,y:15),id:eaf75740-aaba-11ec-a3d2-bd8ccacbecdb,panelIndex:b43e0a6e-2694-4cbe-a364-523cbddc1382,type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'14935429-0f2d-465f-bf17-1b7341b236c4',w:24,x:24,y:15),id:'2ebd58d0-aabb-11ec-a3d2-bd8ccacbecdb',panelIndex:'14935429-0f2d-465f-bf17-1b7341b236c4',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:db4165b9-ba8e-44b2-9dfe-fef124c998df,w:24,x:0,y:30),id:'42f63240-aabb-11ec-a3d2-bd8ccacbecdb',panelIndex:db4165b9-ba8e-44b2-9dfe-fef124c998df,type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'0a9b1447-fc50-4470-a11e-5c840840ea91',w:24,x:24,y:30),id:'6121bcd0-aabb-11ec-a3d2-bd8ccacbecdb',panelIndex:'0a9b1447-fc50-4470-a11e-5c840840ea91',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'390b08b2-83e6-420d-bf24-327038590a8b',w:24,x:0,y:45),id:'6d611d10-aabb-11ec-a3d2-bd8ccacbecdb',panelIndex:'390b08b2-83e6-420d-bf24-327038590a8b',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:d978f4b6-f3d3-4be0-b1f2-24afc4901172,w:24,x:24,y:45),id:'83fbb440-aabb-11ec-a3d2-bd8ccacbecdb',panelIndex:d978f4b6-f3d3-4be0-b1f2-24afc4901172,type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'8c03844f-d71a-475e-989a-d5c871213159',w:24,x:0,y:60),id:d3b762e0-aabb-11ec-a3d2-bd8ccacbecdb,panelIndex:'8c03844f-d71a-475e-989a-d5c871213159',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:fc13a493-fef7-4d09-81cf-1a579bcc37b9,w:24,x:24,y:60),id:e36dc170-aabb-11ec-a3d2-bd8ccacbecdb,panelIndex:fc13a493-fef7-4d09-81cf-1a579bcc37b9,type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:f11bb6f6-ad3e-4c46-bf7e-a2116619c4e9,w:24,x:0,y:75),id:f5ca87e0-aabb-11ec-a3d2-bd8ccacbecdb,panelIndex:f11bb6f6-ad3e-4c46-bf7e-a2116619c4e9,type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'92b7eb46-bacc-4d3c-8db1-12603e1dc97e',w:24,x:24,y:75),id:'0759ec30-aabc-11ec-a3d2-bd8ccacbecdb',panelIndex:'92b7eb46-bacc-4d3c-8db1-12603e1dc97e',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'1d11ccd0-a944-4b8d-8ae2-d5538504aec4',w:24,x:0,y:90),id:'1caa9e40-aabc-11ec-a3d2-bd8ccacbecdb',panelIndex:'1d11ccd0-a944-4b8d-8ae2-d5538504aec4',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:b4f83707-a7ab-46e2-a997-ffb897691934,w:24,x:24,y:90),id:'29043d40-aabc-11ec-a3d2-bd8ccacbecdb',panelIndex:b4f83707-a7ab-46e2-a997-ffb897691934,type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'56cceda0-662b-4fd4-8874-cbf0cd56e4eb',w:24,x:0,y:105),id:'34dd1380-aabc-11ec-a3d2-bd8ccacbecdb',panelIndex:'56cceda0-662b-4fd4-8874-cbf0cd56e4eb',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:b7679f49-69a1-4a45-a196-a2ec3cbebf76,w:24,x:24,y:105),id:'45edf900-aabc-11ec-a3d2-bd8ccacbecdb',panelIndex:b7679f49-69a1-4a45-a196-a2ec3cbebf76,type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'7b91fc27-dbd4-49f1-b9ff-79dd66216653',w:24,x:0,y:120),id:'5ee4a8a0-aabc-11ec-a3d2-bd8ccacbecdb',panelIndex:'7b91fc27-dbd4-49f1-b9ff-79dd66216653',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'745ffbad-7b66-4203-aa76-852bf9e02bd7',w:24,x:24,y:120),id:'72195880-aabc-11ec-a3d2-bd8ccacbecdb',panelIndex:'745ffbad-7b66-4203-aa76-852bf9e02bd7',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:e249ff63-f131-4b22-abd4-0e1ad07ec4e6,w:24,x:0,y:135),id:'863e34c0-aabc-11ec-a3d2-bd8ccacbecdb',panelIndex:e249ff63-f131-4b22-abd4-0e1ad07ec4e6,type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'956c9d98-1d11-4189-9a70-10e4ee2c7e05',w:24,x:24,y:135),id:'9a90b0b0-aabc-11ec-a3d2-bd8ccacbecdb',panelIndex:'956c9d98-1d11-4189-9a70-10e4ee2c7e05',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'9a06e6d2-d1eb-4180-9270-5c03dd7b55d4',w:24,x:0,y:150),id:a6ce1520-aabc-11ec-a3d2-bd8ccacbecdb,panelIndex:'9a06e6d2-d1eb-4180-9270-5c03dd7b55d4',type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:d396581b-56b4-4127-aa7a-f882ea11065c,w:24,x:24,y:150),id:b4d22fd0-aabc-11ec-a3d2-bd8ccacbecdb,panelIndex:d396581b-56b4-4127-aa7a-f882ea11065c,type:visualization,version:'7.17.1'),(embeddableConfig:(enhancements:()),gridData:(h:15,i:'24c32bb4-f37c-47d0-803b-dae94e50c80e',w:24,x:0,y:165),id:c17b7570-aabc-11ec-a3d2-bd8ccacbecdb,panelIndex:'24c32bb4-f37c-47d0-803b-dae94e50c80e',type:visualization,version:'7.17.1')),query:(language:kuery,query:''),tags:!('9c153fc0-aaba-11ec-a3d2-bd8ccacbecdb'),timeRestore:!t,title:'Average%20TWAMP%20Measurements%20Dashboard',viewMode:view)";
        model.addAttribute("classActiveSettingsTwamp", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchTwampAverage";
    }

    @GetMapping(value = "/secure/statistics/pieStatistics")
    public String elasticsearchPieStatistics(Model model, HttpSession session, HttpServletRequest request) {

        UrlParameters urlParameters = createUrlParameters(request);
        String elasticsearchURL;

        if (visualOptionsRepository.getLastEntry() != null) {
            if (visualOptionsRepository.getLastEntry().getUserdata().equals(UserData.HIDE)) {
                elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
                        "/app/kibana#/dashboard/wifimon_privacy_dash_06_v0.1?embed=true&_g=(refreshInterval:(display:'30+seconds',pause:!f,section:1,value:30000),time:(from:now%2Fd,mode:quick,to:now%2Fd))&_a=(description:'',filters:!(),options:(darkTheme:!f),panels:!((col:7,id:wifimon_vis_36_v0.1,panelIndex:1,row:1,size_x:6,size_y:3,type:visualization),(col:1,id:wifimon_vis_35_v0.1,panelIndex:3,row:1,size_x:6,size_y:3,type:visualization)),query:(" + urlParameters.getQueryFilter() + "),timeRestore:!t,title:'WiFiMon:+Pie+statistics+%5Bv0.1-privacy%5D',uiState:(),viewMode:view)";
            } else {
                elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
                        "/app/kibana#/dashboard/wifimon_dash_06_v0.1?embed=true&_g=(refreshInterval:(display:'30+seconds',pause:!f,section:1,value:30000),time:(from:now%2Fd,mode:quick,to:now%2Fd))&_a=(description:'',filters:!(),options:(darkTheme:!f),panels:!((col:1,id:wifimon_vis_36_v0.1,panelIndex:1,row:4,size_x:6,size_y:3,type:visualization),(col:7,id:wifimon_vis_37_v0.1,panelIndex:2,row:1,size_x:6,size_y:3,type:visualization),(col:1,id:wifimon_vis_35_v0.1,panelIndex:3,row:1,size_x:6,size_y:3,type:visualization),(col:7,id:wifimon_vis_45_v0.1,panelIndex:4,row:1,size_x:6,size_y:3,type:visualization)),query:(" + urlParameters.getQueryFilter() + "),timeRestore:!t,title:'WiFiMon:+Pie+statistics+%5Bv0.1%5D',uiState:(),viewMode:view)";
            }
        } else {
            elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
                    "/app/kibana#/dashboard/wifimon_dash_06_v0.1?embed=true&_g=(refreshInterval:(display:'30+seconds',pause:!f,section:1,value:30000),time:(from:now%2Fd,mode:quick,to:now%2Fd))&_a=(description:'',filters:!(),options:(darkTheme:!f),panels:!((col:1,id:wifimon_vis_36_v0.1,panelIndex:1,row:4,size_x:6,size_y:3,type:visualization),(col:7,id:wifimon_vis_37_v0.1,panelIndex:2,row:1,size_x:6,size_y:3,type:visualization),(col:1,id:wifimon_vis_35_v0.1,panelIndex:3,row:1,size_x:6,size_y:3,type:visualization),(col:7,id:wifimon_vis_45_v0.1,panelIndex:4,row:1,size_x:6,size_y:3,type:visualization)),query:(" + urlParameters.getQueryFilter() + "),timeRestore:!t,title:'WiFiMon:+Pie+statistics+%5Bv0.1%5D',uiState:(),viewMode:view)";
        }

        model.addAttribute("classActiveSettingsElasticStatistics", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchPieStatistics";
    }

    @GetMapping(value = "/secure/statistics/tableStatistics")
    public String elasticsearchTableStatistics(Model model, HttpSession session, HttpServletRequest request) {

        String elasticsearchURL;

        elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
                        "/app/kibana#/dashboard/wifimon_dash_07_v0.1?embed=true&_g=(refreshInterval:(pause:!f,value:30000),time:(from:now%2Fd,mode:quick,to:now%2Fd))&_a=(description:'',filters:!(),fullScreenMode:!f,options:(darkTheme:!f),panels:!((embeddableConfig:(vis:(params:(sort:(columnIndex:!n,direction:!n)))),gridData:(h:15,i:'1',w:24,x:0,y:0),id:wifimon_vis_02_v0.1,panelIndex:'1',type:visualization,version:'6.8.3'),(embeddableConfig:(vis:(params:(sort:(columnIndex:!n,direction:!n)))),gridData:(h:15,i:'2',w:24,x:24,y:0),id:wifimon_vis_03_v0.1,panelIndex:'2',type:visualization,version:'6.8.3'),(embeddableConfig:(vis:(params:(sort:(columnIndex:!n,direction:!n)))),gridData:(h:15,i:'5',w:24,x:0,y:15),id:wifimon_vis_10_v0.1,panelIndex:'5',type:visualization,version:'6.8.3'),(embeddableConfig:(vis:(params:(sort:(columnIndex:!n,direction:!n)))),gridData:(h:15,i:'6',w:24,x:24,y:15),id:wifimon_vis_09_v0.1,panelIndex:'6',type:visualization,version:'6.8.3'),(embeddableConfig:(vis:(params:(sort:(columnIndex:!n,direction:!n)))),gridData:(h:15,i:'7',w:24,x:0,y:30),id:wifimon_vis_08_v0.1,panelIndex:'7',type:visualization,version:'6.8.3')),query:(language:lucene,query:(match_all:())),timeRestore:!t,title:'WiFiMon:+Table+statistics+%5Bv0.1%5D',viewMode:view)";

        model.addAttribute("classActiveSettingsElasticStatistics", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchTableStatistics";
    }

    @GetMapping(value = "/secure/maps/clients")
    public String elasticsearchMap(Model model, HttpSession session, HttpServletRequest request) {

        UrlParameters urlParameters = createUrlParameters(request);

        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
                "/app/kibana#/dashboard/wifimon_dash_08_v0.1?embed=true&_g=(refreshInterval:(display:'30+seconds',pause:!f,section:1,value:30000),time:(from:now%2Fd,mode:quick,to:now%2Fd))&_a=(description:'',filters:!(),options:(darkTheme:!f),panels:!((col:1,id:wifimon_vis_07_v0.1,panelIndex:1,row:1,size_x:12,size_y:6,type:visualization)),query:(" + urlParameters.getQueryFilter() + "),timeRestore:!t,title:'WiFiMon:+Measurements+per+Client+map+%5Bv0.1%5D',uiState:(),viewMode:view)";
        model.addAttribute("classActiveSettingsElasticMap", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchMeasurementsMap";
    }

    @GetMapping(value = "/secure/maps/aps")
    public String elasticsearchApMap(Model model, HttpSession session, HttpServletRequest request) {

        UrlParameters urlParameters = createUrlParameters(request);

        String elasticsearchURL = environment.getProperty(KIBANA_PROTOCOL) + "://" + environment.getProperty(SERVER_HOST_NAME) + ":" + environment.getProperty(KIBANA_PORT) +
                "/app/kibana#/dashboard/wifimon_dash_09_v0.1?embed=true&_g=(refreshInterval:(display:'30+seconds',pause:!f,section:1,value:30000),time:(from:now%2Fd,mode:quick,to:now%2Fd))&_a=(description:'',filters:!(),options:(darkTheme:!f),panels:!((col:1,id:wifimon_vis_42_v0.1,panelIndex:1,row:1,size_x:12,size_y:6,type:visualization)),query:(" + urlParameters.getQueryFilter() + "),timeRestore:!t,title:'WiFiMon:+Measurements+per+AP+map+%5Bv0.1%5D',uiState:(),viewMode:view)";
        model.addAttribute("classActiveSettingsElasticMap", CLASS_SETTINGS_STATE);
        model.addAttribute(ELASTICSEARCH_URL, elasticsearchURL);
        return "secure/elasticsearchApMap";
    }

    @GetMapping(value = "/secure/guide")
    public String guide(Model model, HttpSession session) {
        model.addAttribute("classActiveSettingsGuide", CLASS_SETTINGS_STATE);
        return "secure/guide";
    }

    @GetMapping(value = "/secure/help")
    public String help(Model model, HttpSession session) {
        model.addAttribute("classActiveSettingsHelp", CLASS_SETTINGS_STATE);
        return "secure/help";
    }

    private UrlParameters createUrlParameters(HttpServletRequest request) {
        UrlParameters urlParameters = new UrlParameters();
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        String currentUserEmail = authentication.getName();
        String currentUserRole = userRepository.getRoleByEmail(currentUserEmail);
        String currentUserIp = request.getRemoteAddr();
        String queryFilter;

        if (visualOptionsRepository.getLastEntry() != null && currentUserIp != null && !currentUserRole.equals("ADMIN")) {
            if (visualOptionsRepository.getLastEntry().getUservisualoption().equals(UserVisualOption.MATCH_USERNAME)) {
                queryFilter = "query_string:(analyze_wildcard:!t,query:'username:*" + currentUserEmail + "*')";
            } else if (visualOptionsRepository.getLastEntry().getUservisualoption().equals(UserVisualOption.MATCH_IP)) {
                queryFilter = "query_string:(analyze_wildcard:!t,query:'clientIp:*" + currentUserIp + "*')";
            } else {
                queryFilter = "match_all:()";
            }
        } else {
            queryFilter = "match_all:()";
        }
        urlParameters.setCurrentUserRole(currentUserRole);
        urlParameters.setQueryFilter(queryFilter);
        return urlParameters;
    }
}
